@@ -1,6 +1,8 @@
 /*
  * contrib/btree_gist/btree_inet.c
  */
+#include "postgres.h"
+
 #include "btree_gist.h"
 #include "btree_utils_num.h"
 #include "utils/builtins.h"
@@ -34,34 +36,34 @@ Datum		gbt_inet_same(PG_FUNCTION_ARGS);
 static bool
 gbt_inetgt(const void *a, const void *b)
 {
-	return (*((double *) a) > *((double *) b));
+	return (*((const double *) a) > *((const double *) b));
 }
 static bool
 gbt_inetge(const void *a, const void *b)
 {
-	return (*((double *) a) >= *((double *) b));
+	return (*((const double *) a) >= *((const double *) b));
 }
 static bool
 gbt_ineteq(const void *a, const void *b)
 {
-	return (*((double *) a) == *((double *) b));
+	return (*((const double *) a) == *((const double *) b));
 }
 static bool
 gbt_inetle(const void *a, const void *b)
 {
-	return (*((double *) a) <= *((double *) b));
+	return (*((const double *) a) <= *((const double *) b));
 }
 static bool
 gbt_inetlt(const void *a, const void *b)
 {
-	return (*((double *) a) < *((double *) b));
+	return (*((const double *) a) < *((const double *) b));
 }
 
 static int
 gbt_inetkey_cmp(const void *a, const void *b)
 {
-	inetKEY    *ia = (inetKEY *) (((Nsrt *) a)->t);
-	inetKEY    *ib = (inetKEY *) (((Nsrt *) b)->t);
+	inetKEY    *ia = (inetKEY *) (((const Nsrt *) a)->t);
+	inetKEY    *ib = (inetKEY *) (((const Nsrt *) b)->t);
 
 	if (ia->lower == ib->lower)
 	{
@@ -103,9 +105,11 @@ gbt_inet_compress(PG_FUNCTION_ARGS)
 	if (entry->leafkey)
 	{
 		inetKEY    *r = (inetKEY *) palloc(sizeof(inetKEY));
+		bool		failure = false;
 
 		retval = palloc(sizeof(GISTENTRY));
-		r->lower = convert_network_to_scalar(entry->key, INETOID);
+		r->lower = convert_network_to_scalar(entry->key, INETOID, &failure);
+		Assert(!failure);
 		r->upper = r->lower;
 		gistentryinit(*retval, PointerGetDatum(r),
 					  entry->rel, entry->page,
@@ -122,13 +126,18 @@ Datum
 gbt_inet_consistent(PG_FUNCTION_ARGS)
 {
 	GISTENTRY  *entry = (GISTENTRY *) PG_GETARG_POINTER(0);
-	double		query = convert_network_to_scalar(PG_GETARG_DATUM(1), INETOID);
+	Datum		dquery = PG_GETARG_DATUM(1);
 	StrategyNumber strategy = (StrategyNumber) PG_GETARG_UINT16(2);
 
 	/* Oid		subtype = PG_GETARG_OID(3); */
 	bool	   *recheck = (bool *) PG_GETARG_POINTER(4);
 	inetKEY    *kkk = (inetKEY *) DatumGetPointer(entry->key);
 	GBT_NUMKEY_R key;
+	double		query;
+	bool		failure = false;
+
+	query = convert_network_to_scalar(dquery, INETOID, &failure);
+	Assert(!failure);
 
 	/* All cases served by this function are inexact */
 	*recheck = true;
