@@ -1,6 +1,8 @@
 /*
- * $PostgreSQL: pgsql/contrib/btree_gist/btree_macaddr.c,v 1.8 2009/06/11 14:48:50 momjian Exp $
+ * contrib/btree_gist/btree_macaddr.c
  */
+#include "postgres.h"
+
 #include "btree_gist.h"
 #include "btree_utils_num.h"
 #include "utils/builtins.h"
@@ -63,13 +65,15 @@ gbt_macadlt(const void *a, const void *b)
 static int
 gbt_macadkey_cmp(const void *a, const void *b)
 {
-	return DatumGetInt32(
-						 DirectFunctionCall2(
-											 macaddr_cmp,
-										PointerGetDatum(&((Nsrt *) a)->t[0]),
-										 PointerGetDatum(&((Nsrt *) b)->t[0])
-											 )
-		);
+	macKEY	   *ia = (macKEY *) (((const Nsrt *) a)->t);
+	macKEY	   *ib = (macKEY *) (((const Nsrt *) b)->t);
+	int			res;
+
+	res = DatumGetInt32(DirectFunctionCall2(macaddr_cmp, MacaddrPGetDatum(&ia->lower), MacaddrPGetDatum(&ib->lower)));
+	if (res == 0)
+		return DatumGetInt32(DirectFunctionCall2(macaddr_cmp, MacaddrPGetDatum(&ia->upper), MacaddrPGetDatum(&ib->upper)));
+
+	return res;
 }
 
 
@@ -82,7 +86,8 @@ static const gbtree_ninfo tinfo =
 	gbt_macadeq,
 	gbt_macadle,
 	gbt_macadlt,
-	gbt_macadkey_cmp
+	gbt_macadkey_cmp,
+	NULL
 };
 
 
