@@ -1,8 +1,11 @@
 /*
- * $PostgreSQL: pgsql/contrib/btree_gist/btree_ts.c,v 1.19 2010/02/26 02:00:32 momjian Exp $
+ * contrib/btree_gist/btree_ts.c
  */
+#include "postgres.h"
+
 #include "btree_gist.h"
 #include "btree_utils_num.h"
+#include "utils/builtins.h"
 #include "utils/datetime.h"
 
 typedef struct
@@ -19,7 +22,9 @@ PG_FUNCTION_INFO_V1(gbt_tstz_compress);
 PG_FUNCTION_INFO_V1(gbt_ts_union);
 PG_FUNCTION_INFO_V1(gbt_ts_picksplit);
 PG_FUNCTION_INFO_V1(gbt_ts_consistent);
+PG_FUNCTION_INFO_V1(gbt_ts_distance);
 PG_FUNCTION_INFO_V1(gbt_tstz_consistent);
+PG_FUNCTION_INFO_V1(gbt_tstz_distance);
 PG_FUNCTION_INFO_V1(gbt_ts_penalty);
 PG_FUNCTION_INFO_V1(gbt_ts_same);
 
@@ -28,7 +33,9 @@ Datum		gbt_tstz_compress(PG_FUNCTION_ARGS);
 Datum		gbt_ts_union(PG_FUNCTION_ARGS);
 Datum		gbt_ts_picksplit(PG_FUNCTION_ARGS);
 Datum		gbt_ts_consistent(PG_FUNCTION_ARGS);
+Datum		gbt_ts_distance(PG_FUNCTION_ARGS);
 Datum		gbt_tstz_consistent(PG_FUNCTION_ARGS);
+Datum		gbt_tstz_distance(PG_FUNCTION_ARGS);
 Datum		gbt_ts_penalty(PG_FUNCTION_ARGS);
 Datum		gbt_ts_same(PG_FUNCTION_ARGS);
 
@@ -99,8 +106,8 @@ gbt_tslt(const void *a, const void *b)
 static int
 gbt_tskey_cmp(const void *a, const void *b)
 {
-	tsKEY	   *ia = (tsKEY *) (((Nsrt *) a)->t);
-	tsKEY	   *ib = (tsKEY *) (((Nsrt *) b)->t);
+	tsKEY	   *ia = (tsKEY *) (((const Nsrt *) a)->t);
+	tsKEY	   *ib = (tsKEY *) (((const Nsrt *) b)->t);
 	int			res;
 
 	res = DatumGetInt32(DirectFunctionCall2(timestamp_cmp, TimestampGetDatumFast(ia->lower), TimestampGetDatumFast(ib->lower)));
@@ -110,6 +117,22 @@ gbt_tskey_cmp(const void *a, const void *b)
 	return res;
 }
 
+static float8
+gbt_ts_dist(const void *a, const void *b)
+{
+	const Timestamp *aa = (const Timestamp *) a;
+	const Timestamp *bb = (const Timestamp *) b;
+	Interval   *i;
+
+	if (TIMESTAMP_NOT_FINITE(*aa) || TIMESTAMP_NOT_FINITE(*bb))
+		return get_float8_infinity();
+
+	i = DatumGetIntervalP(DirectFunctionCall2(timestamp_mi,
+											  TimestampGetDatumFast(*aa),
+											  TimestampGetDatumFast(*bb)));
+	return (float8) Abs(INTERVAL_TO_SEC(i));
+}
+
 
 static const gbtree_ninfo tinfo =
 {
@@ -120,10 +143,70 @@ static const gbtree_ninfo tinfo =
 	gbt_tseq,
 	gbt_tsle,
 	gbt_tslt,
-	gbt_tskey_cmp
+	gbt_tskey_cmp,
+	gbt_ts_dist
 };
 
 
+PG_FUNCTION_INFO_V1(ts_dist);
+Datum		ts_dist(PG_FUNCTION_ARGS);
+Datum
+ts_dist(PG_FUNCTION_ARGS)
+{
+	Timestamp	a = PG_GETARG_TIMESTAMP(0);
+	Timestamp	b = PG_GETARG_TIMESTAMP(1);
+	Interval   *r;
+
+	if (TIMESTAMP_NOT_FINITE(a) || TIMESTAMP_NOT_FINITE(b))
+	{
+		Interval   *p = palloc(sizeof(Interval));
+
+		p->day = INT_MAX;
+		p->month = INT_MAX;
+#ifdef HAVE_INT64_TIMESTAMP
+		p->time = INT64CONST(0x7FFFFFFFFFFFFFFF);
+#else
+		p->time = DBL_MAX;
+#endif
+		PG_RETURN_INTERVAL_P(p);
+	}
+	else
+		r = DatumGetIntervalP(DirectFunctionCall2(timestamp_mi,
+												  PG_GETARG_DATUM(0),
+												  PG_GETARG_DATUM(1)));
+	PG_RETURN_INTERVAL_P(abs_interval(r));
+}
+
+PG_FUNCTION_INFO_V1(tstz_dist);
+Datum		tstz_dist(PG_FUNCTION_ARGS);
+Datum
+tstz_dist(PG_FUNCTION_ARGS)
+{
+	TimestampTz a = PG_GETARG_TIMESTAMPTZ(0);
+	TimestampTz b = PG_GETARG_TIMESTAMPTZ(1);
+	Interval   *r;
+
+	if (TIMESTAMP_NOT_FINITE(a) || TIMESTAMP_NOT_FINITE(b))
+	{
+		Interval   *p = palloc(sizeof(Interval));
+
+		p->day = INT_MAX;
+		p->month = INT_MAX;
+#ifdef HAVE_INT64_TIMESTAMP
+		p->time = INT64CONST(0x7FFFFFFFFFFFFFFF);
+#else
+		p->time = DBL_MAX;
+#endif
+		PG_RETURN_INTERVAL_P(p);
+	}
+
+	r = DatumGetIntervalP(DirectFunctionCall2(timestamp_mi,
+											  PG_GETARG_DATUM(0),
+											  PG_GETARG_DATUM(1)));
+	PG_RETURN_INTERVAL_P(abs_interval(r));
+}
+
+
 /**************************************************
  * timestamp ops
  **************************************************/
@@ -197,6 +280,24 @@ gbt_ts_consistent(PG_FUNCTION_ARGS)
 		);
 }
 
+Datum
+gbt_ts_distance(PG_FUNCTION_ARGS)
+{
+	GISTENTRY  *entry = (GISTENTRY *) PG_GETARG_POINTER(0);
+	Timestamp	query = PG_GETARG_TIMESTAMP(1);
+
+	/* Oid		subtype = PG_GETARG_OID(3); */
+	tsKEY	   *kkk = (tsKEY *) DatumGetPointer(entry->key);
+	GBT_NUMKEY_R key;
+
+	key.lower = (GBT_NUMKEY *) &kkk->lower;
+	key.upper = (GBT_NUMKEY *) &kkk->upper;
+
+	PG_RETURN_FLOAT8(
+			gbt_num_distance(&key, (void *) &query, GIST_LEAF(entry), &tinfo)
+		);
+}
+
 Datum
 gbt_tstz_consistent(PG_FUNCTION_ARGS)
 {
@@ -222,6 +323,26 @@ gbt_tstz_consistent(PG_FUNCTION_ARGS)
 		);
 }
 
+Datum
+gbt_tstz_distance(PG_FUNCTION_ARGS)
+{
+	GISTENTRY  *entry = (GISTENTRY *) PG_GETARG_POINTER(0);
+	TimestampTz query = PG_GETARG_TIMESTAMPTZ(1);
+
+	/* Oid		subtype = PG_GETARG_OID(3); */
+	char	   *kkk = (char *) DatumGetPointer(entry->key);
+	GBT_NUMKEY_R key;
+	Timestamp	qqq;
+
+	key.lower = (GBT_NUMKEY *) &kkk[0];
+	key.upper = (GBT_NUMKEY *) &kkk[MAXALIGN(tinfo.size)];
+	qqq = tstz_to_ts_gmt(query);
+
+	PG_RETURN_FLOAT8(
+			  gbt_num_distance(&key, (void *) &qqq, GIST_LEAF(entry), &tinfo)
+		);
+}
+
 
 Datum
 gbt_ts_union(PG_FUNCTION_ARGS)
@@ -245,7 +366,6 @@ gbt_ts_union(PG_FUNCTION_ARGS)
 Datum
 gbt_ts_penalty(PG_FUNCTION_ARGS)
 {
-
 	tsKEY	   *origentry = (tsKEY *) DatumGetPointer(((GISTENTRY *) PG_GETARG_POINTER(0))->key);
 	tsKEY	   *newentry = (tsKEY *) DatumGetPointer(((GISTENTRY *) PG_GETARG_POINTER(1))->key);
 	float	   *result = (float *) PG_GETARG_POINTER(2);
@@ -254,7 +374,7 @@ gbt_ts_penalty(PG_FUNCTION_ARGS)
 				newdbl[2];
 
 	/*
-	 * We are allways using "double" timestamps here. Precision should be good
+	 * We are always using "double" timestamps here. Precision should be good
 	 * enough.
 	 */
 	orgdbl[0] = ((double) origentry->lower);
