@@ -5,9 +5,14 @@
 
 #include <ctype.h>
 
+#include "access/htup_details.h"
 #include "catalog/pg_type.h"
 #include "funcapi.h"
+#include "lib/stringinfo.h"
 #include "libpq/pqformat.h"
+#include "utils/builtins.h"
+#include "utils/json.h"
+#include "utils/jsonapi.h"
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
 #include "utils/typcache.h"
@@ -37,7 +42,7 @@ typedef struct
 do { \
 		if ( state->cur - state->word + 1 >= state->wordlen ) \
 		{ \
-				int4 clen = state->cur - state->word; \
+				int32 clen = state->cur - state->word; \
 				state->wordlen *= 2; \
 				state->word = (char*)repalloc( (void*)state->word, state->wordlen ); \
 				state->cur = state->word + clen; \
@@ -75,7 +80,7 @@ get_val(HSParser *state, bool ignoreeq, bool *escaped)
 			}
 			else if (*(state->ptr) == '=' && !ignoreeq)
 			{
-				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int4) (state->ptr - state->begin));
+				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int32) (state->ptr - state->begin));
 			}
 			else if (*(state->ptr) == '\\')
 			{
@@ -164,8 +169,6 @@ get_val(HSParser *state, bool ignoreeq, bool *escaped)
 
 		state->ptr++;
 	}
-
-	return false;
 }
 
 #define WKEY	0
@@ -216,7 +219,7 @@ parse_hstore(HSParser *state)
 			}
 			else if (!isspace((unsigned char) *(state->ptr)))
 			{
-				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int4) (state->ptr - state->begin));
+				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int32) (state->ptr - state->begin));
 			}
 		}
 		else if (st == WGT)
@@ -231,7 +234,7 @@ parse_hstore(HSParser *state)
 			}
 			else
 			{
-				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int4) (state->ptr - state->begin));
+				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int32) (state->ptr - state->begin));
 			}
 		}
 		else if (st == WVAL)
@@ -264,7 +267,7 @@ parse_hstore(HSParser *state)
 			}
 			else if (!isspace((unsigned char) *(state->ptr)))
 			{
-				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int4) (state->ptr - state->begin));
+				elog(ERROR, "Syntax error near '%c' at position %d", *(state->ptr), (int32) (state->ptr - state->begin));
 			}
 		}
 		else
@@ -305,7 +308,7 @@ comparePairs(const void *a, const void *b)
  * and (b) who knows whether they might be needed by some caller.
  */
 int
-hstoreUniquePairs(Pairs *a, int4 l, int4 *buflen)
+hstoreUniquePairs(Pairs *a, int32 l, int32 *buflen)
 {
 	Pairs	   *ptr,
 			   *res;
@@ -336,7 +339,8 @@ hstoreUniquePairs(Pairs *a, int4 l, int4 *buflen)
 		{
 			*buflen += res->keylen + ((res->isnull) ? 0 : res->vallen);
 			res++;
-			memcpy(res, ptr, sizeof(Pairs));
+			if (res != ptr)
+				memcpy(res, ptr, sizeof(Pairs));
 		}
 
 		ptr++;
@@ -368,14 +372,14 @@ hstoreCheckValLen(size_t len)
 
 
 HStore *
-hstorePairs(Pairs *pairs, int4 pcount, int4 buflen)
+hstorePairs(Pairs *pairs, int32 pcount, int32 buflen)
 {
 	HStore	   *out;
 	HEntry	   *entry;
 	char	   *ptr;
 	char	   *buf;
-	int4		len;
-	int4		i;
+	int32		len;
+	int32		i;
 
 	len = CALCDATASIZE(pcount, buflen);
 	out = palloc(len);
@@ -403,7 +407,7 @@ Datum
 hstore_in(PG_FUNCTION_ARGS)
 {
 	HSParser	state;
-	int4		buflen;
+	int32		buflen;
 	HStore	   *out;
 
 	state.begin = PG_GETARG_CSTRING(0);
@@ -423,11 +427,11 @@ Datum		hstore_recv(PG_FUNCTION_ARGS);
 Datum
 hstore_recv(PG_FUNCTION_ARGS)
 {
-	int4		buflen;
+	int32		buflen;
 	HStore	   *out;
 	Pairs	   *pairs;
-	int4		i;
-	int4		pcount;
+	int32		i;
+	int32		pcount;
 	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
 
 	pcount = pq_getmsgint(buf, 4);
@@ -524,7 +528,7 @@ Datum		hstore_from_arrays(PG_FUNCTION_ARGS);
 Datum
 hstore_from_arrays(PG_FUNCTION_ARGS)
 {
-	int4		buflen;
+	int32		buflen;
 	HStore	   *out;
 	Pairs	   *pairs;
 	Datum	   *key_datums;
@@ -645,7 +649,7 @@ hstore_from_array(PG_FUNCTION_ARGS)
 	ArrayType  *in_array = PG_GETARG_ARRAYTYPE_P(0);
 	int			ndims = ARR_NDIM(in_array);
 	int			count;
-	int4		buflen;
+	int32		buflen;
 	HStore	   *out;
 	Pairs	   *pairs;
 	Datum	   *in_datums;
@@ -757,7 +761,7 @@ Datum
 hstore_from_record(PG_FUNCTION_ARGS)
 {
 	HeapTupleHeader rec;
-	int4		buflen;
+	int32		buflen;
 	HStore	   *out;
 	Pairs	   *pairs;
 	Oid			tupType;
@@ -1231,3 +1235,108 @@ hstore_send(PG_FUNCTION_ARGS)
 
 	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
 }
+
+
+/*
+ * hstore_to_json_loose
+ *
+ * This is a heuristic conversion to json which treats
+ * 't' and 'f' as booleans and strings that look like numbers as numbers,
+ * as long as they don't start with a leading zero followed by another digit
+ * (think zip codes or phone numbers starting with 0).
+ */
+PG_FUNCTION_INFO_V1(hstore_to_json_loose);
+Datum		hstore_to_json_loose(PG_FUNCTION_ARGS);
+Datum
+hstore_to_json_loose(PG_FUNCTION_ARGS)
+{
+	HStore	   *in = PG_GETARG_HS(0);
+	int			i;
+	int			count = HS_COUNT(in);
+	char	   *base = STRPTR(in);
+	HEntry	   *entries = ARRPTR(in);
+	StringInfoData tmp,
+				dst;
+
+	if (count == 0)
+		PG_RETURN_TEXT_P(cstring_to_text_with_len("{}",2));
+
+	initStringInfo(&tmp);
+	initStringInfo(&dst);
+
+	appendStringInfoChar(&dst, '{');
+
+	for (i = 0; i < count; i++)
+	{
+		resetStringInfo(&tmp);
+		appendBinaryStringInfo(&tmp, HS_KEY(entries, base, i), HS_KEYLEN(entries, i));
+		escape_json(&dst, tmp.data);
+		appendStringInfoString(&dst, ": ");
+		if (HS_VALISNULL(entries, i))
+			appendStringInfoString(&dst, "null");
+		/* guess that values of 't' or 'f' are booleans */
+		else if (HS_VALLEN(entries, i) == 1 && *(HS_VAL(entries, base, i)) == 't')
+			appendStringInfoString(&dst, "true");
+		else if (HS_VALLEN(entries, i) == 1 && *(HS_VAL(entries, base, i)) == 'f')
+			appendStringInfoString(&dst, "false");
+		else
+		{
+			resetStringInfo(&tmp);
+			appendBinaryStringInfo(&tmp, HS_VAL(entries, base, i), HS_VALLEN(entries, i));
+			if (IsValidJsonNumber(tmp.data, tmp.len))
+				appendBinaryStringInfo(&dst, tmp.data, tmp.len);
+			else
+				escape_json(&dst, tmp.data);
+		}
+
+		if (i + 1 != count)
+			appendStringInfoString(&dst, ", ");
+	}
+	appendStringInfoChar(&dst, '}');
+
+	PG_RETURN_TEXT_P(cstring_to_text(dst.data));
+}
+
+PG_FUNCTION_INFO_V1(hstore_to_json);
+Datum		hstore_to_json(PG_FUNCTION_ARGS);
+Datum
+hstore_to_json(PG_FUNCTION_ARGS)
+{
+	HStore	   *in = PG_GETARG_HS(0);
+	int			i;
+	int			count = HS_COUNT(in);
+	char	   *base = STRPTR(in);
+	HEntry	   *entries = ARRPTR(in);
+	StringInfoData tmp,
+				dst;
+
+	if (count == 0)
+		PG_RETURN_TEXT_P(cstring_to_text_with_len("{}",2));
+
+	initStringInfo(&tmp);
+	initStringInfo(&dst);
+
+	appendStringInfoChar(&dst, '{');
+
+	for (i = 0; i < count; i++)
+	{
+		resetStringInfo(&tmp);
+		appendBinaryStringInfo(&tmp, HS_KEY(entries, base, i), HS_KEYLEN(entries, i));
+		escape_json(&dst, tmp.data);
+		appendStringInfoString(&dst, ": ");
+		if (HS_VALISNULL(entries, i))
+			appendStringInfoString(&dst, "null");
+		else
+		{
+			resetStringInfo(&tmp);
+			appendBinaryStringInfo(&tmp, HS_VAL(entries, base, i), HS_VALLEN(entries, i));
+			escape_json(&dst, tmp.data);
+		}
+
+		if (i + 1 != count)
+			appendStringInfoString(&dst, ", ");
+	}
+	appendStringInfoChar(&dst, '}');
+
+	PG_RETURN_TEXT_P(cstring_to_text(dst.data));
+}
