@@ -57,6 +57,7 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 	bool		got_date_is_int = false;
 	bool		got_float8_pass_by_value = false;
 	bool		got_data_checksum_version = false;
+	bool		got_cluster_state = false;
 	char	   *lc_collate = NULL;
 	char	   *lc_ctype = NULL;
 	char	   *lc_monetary = NULL;
@@ -110,6 +111,65 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 	pg_putenv("LC_ALL", NULL);
 	pg_putenv("LC_MESSAGES", "C");
 
+	/*
+	 * Check for clean shutdown
+	 */
+	if (!live_check || cluster == &new_cluster)
+	{
+		/* only pg_controldata outputs the cluster state */
+		snprintf(cmd, sizeof(cmd), SYSTEMQUOTE "\"%s/pg_controldata\" \"%s\"" SYSTEMQUOTE,
+				 cluster->bindir, cluster->pgdata);
+		fflush(stdout);
+		fflush(stderr);
+
+		if ((output = popen(cmd, "r")) == NULL)
+			pg_log(PG_FATAL, "could not get control data using %s: %s\n",
+					 cmd, strerror(errno));
+
+		/* we have the result of cmd in "output". so parse it line by line now */
+		while (fgets(bufin, sizeof(bufin), output))
+		{
+			if ((p = strstr(bufin, "Database cluster state:")) != NULL)
+			{
+				p = strchr(p, ':');
+
+				if (p == NULL || strlen(p) <= 1)
+					pg_log(PG_FATAL, "%d: database cluster state problem\n", __LINE__);
+
+				p++;				/* remove ':' char */
+
+				/*
+				 * We checked earlier for a postmaster lock file, and if we found
+				 * one, we tried to start/stop the server to replay the WAL.  However,
+				 * pg_ctl -m immediate doesn't leave a lock file, but does require
+				 * WAL replay, so we check here that the server was shut down cleanly,
+				 * from the controldata perspective.
+				 */
+				/* remove leading spaces */
+				while (*p == ' ')
+					p++;
+				if (strcmp(p, "shut down\n") != 0)
+				{
+					if (cluster == &old_cluster)
+						pg_log(PG_FATAL, "The source cluster was not shut down cleanly.\n");
+					else
+						pg_log(PG_FATAL, "The target cluster was not shut down cleanly.\n");
+				}
+				got_cluster_state = true;
+			}
+		}
+
+		pclose(output);
+
+		if (!got_cluster_state)
+		{
+			if (cluster == &old_cluster)
+				pg_log(PG_FATAL, "The source cluster lacks cluster state information:\n");
+			else
+				pg_log(PG_FATAL, "The target cluster lacks cluster state information:\n");
+		}
+	}
+
 	snprintf(cmd, sizeof(cmd), SYSTEMQUOTE "\"%s/%s \"%s\"" SYSTEMQUOTE,
 			 cluster->bindir,
 			 live_check ? "pg_controldata\"" : "pg_resetxlog\" -n",
@@ -229,7 +289,7 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 				pg_log(PG_FATAL, "%d: controldata retrieval problem\n", __LINE__);
 
 			p++;				/* removing ':' char */
-			cluster->controldata.chkpnt_tli = str2uint(p);
+			tli = str2uint(p);
 			got_tli = true;
 		}
 		else if ((p = strstr(bufin, "Latest checkpoint's NextXID:")) != NULL)
@@ -479,11 +539,11 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 	 * Before 9.3, pg_resetxlog reported the xlogid and segno of the first log
 	 * file after reset as separate lines. Starting with 9.3, it reports the
 	 * WAL file name. If the old cluster is older than 9.3, we construct the
-	 * WAL file name from the xlogid and segno.
+	 * WAL file name from the tli, xlogid, and segno.
 	 */
 	if (GET_MAJOR_VERSION(cluster->major_version) <= 902)
 	{
-		if (got_log_id && got_log_seg)
+		if (got_tli && got_log_id && got_log_seg)
 		{
 			snprintf(cluster->controldata.nextxlogfile, 25, "%08X%08X%08X",
 					 tli, logid, segno);
@@ -497,7 +557,6 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 		(!got_oldestmulti &&
 		 cluster->controldata.cat_ver >= MULTIXACT_FORMATCHANGE_CAT_VER) ||
 		(!live_check && !got_nextxlogfile) ||
-		!got_tli ||
 		!got_align || !got_blocksz || !got_largesz || !got_walsz ||
 		!got_walseg || !got_ident || !got_index || !got_toast ||
 		!got_date_is_int || !got_float8_pass_by_value || !got_data_checksum_version)
@@ -525,9 +584,6 @@ get_control_data(ClusterInfo *cluster, bool live_check)
 		if (!live_check && !got_nextxlogfile)
 			pg_log(PG_REPORT, "  first WAL segment after reset\n");
 
-		if (!got_tli)
-			pg_log(PG_REPORT, "  latest checkpoint timeline ID\n");
-
 		if (!got_align)
 			pg_log(PG_REPORT, "  maximum alignment\n");
 
@@ -589,7 +645,7 @@ check_control_data(ControlData *oldctrl,
 
 	if (oldctrl->largesz == 0 || oldctrl->largesz != newctrl->largesz)
 		pg_log(PG_FATAL,
-			   "old and new pg_controldata maximum relation segement sizes are invalid or do not match\n");
+			   "old and new pg_controldata maximum relation segment sizes are invalid or do not match\n");
 
 	if (oldctrl->walsz == 0 || oldctrl->walsz != newctrl->walsz)
 		pg_log(PG_FATAL,
