@@ -3,11 +3,11 @@
  *
  *	execution functions
  *
- *	Copyright (c) 2010-2012, PostgreSQL Global Development Group
+ *	Copyright (c) 2010-2013, PostgreSQL Global Development Group
  *	contrib/pg_upgrade/exec.c
  */
 
-#include "postgres.h"
+#include "postgres_fe.h"
 
 #include "pg_upgrade.h"
 
@@ -37,20 +37,28 @@ static int	win32_check_directory_write_permissions(void);
  * If throw_error is true, this raises a PG_FATAL error and pg_upgrade
  * terminates; otherwise it is just reported as PG_REPORT and exec_prog()
  * returns false.
+ *
+ * The code requires it be called first from the primary thread on Windows.
  */
 bool
 exec_prog(const char *log_file, const char *opt_log_file,
 		  bool throw_error, const char *fmt,...)
 {
-	int			result;
+	int			result = 0;
 	int			written;
+
 #define MAXCMDLEN (2 * MAXPGPATH)
 	char		cmd[MAXCMDLEN];
-	mode_t		old_umask = 0;
 	FILE	   *log;
 	va_list		ap;
 
-	old_umask = umask(S_IRWXG | S_IRWXO);
+#ifdef WIN32
+static DWORD       mainThreadId = 0;
+
+	/* We assume we are called from the primary thread first */
+	if (mainThreadId == 0)
+		mainThreadId = GetCurrentThreadId();
+#endif
 
 	written = strlcpy(cmd, SYSTEMQUOTE, sizeof(cmd));
 	va_start(ap, fmt);
@@ -63,34 +71,57 @@ exec_prog(const char *log_file, const char *opt_log_file,
 	if (written >= MAXCMDLEN)
 		pg_log(PG_FATAL, "command too long\n");
 
-	log = fopen_priv(log_file, "a");
+	pg_log(PG_VERBOSE, "%s\n", cmd);
+
+#ifdef WIN32
+	/*
+	 * For some reason, Windows issues a file-in-use error if we write data
+	 * to the log file from a non-primary thread just before we create a
+	 * subprocess that also writes to the same log file.  One fix is to
+	 * sleep for 100ms.  A cleaner fix is to write to the log file _after_
+	 * the subprocess has completed, so we do this only when writing from
+	 * a non-primary thread.  fflush(), running system() twice, and
+	 * pre-creating the file do not see to help.
+	 */
+	if (mainThreadId != GetCurrentThreadId())
+		result = system(cmd);
+#endif
+
+	log = fopen(log_file, "a");
 
 #ifdef WIN32
 	{
-		/* 
-		 * "pg_ctl -w stop" might have reported that the server has
-		 * stopped because the postmaster.pid file has been removed,
-		 * but "pg_ctl -w start" might still be in the process of
-		 * closing and might still be holding its stdout and -l log
-		 * file descriptors open.  Therefore, try to open the log 
-		 * file a few more times.
+		/*
+		 * "pg_ctl -w stop" might have reported that the server has stopped
+		 * because the postmaster.pid file has been removed, but "pg_ctl -w
+		 * start" might still be in the process of closing and might still be
+		 * holding its stdout and -l log file descriptors open.  Therefore,
+		 * try to open the log file a few more times.
 		 */
-		int iter;
+		int			iter;
+
 		for (iter = 0; iter < 4 && log == NULL; iter++)
 		{
 			sleep(1);
-			log = fopen_priv(log_file, "a");
+			log = fopen(log_file, "a");
 		}
 	}
 #endif
 
 	if (log == NULL)
 		pg_log(PG_FATAL, "cannot write to log file %s\n", log_file);
+
 #ifdef WIN32
-	fprintf(log, "\n\n");
+	/* Are we printing "command:" before its output? */
+	if (mainThreadId == GetCurrentThreadId())
+		fprintf(log, "\n\n");
 #endif
-	pg_log(PG_VERBOSE, "%s\n", cmd);
 	fprintf(log, "command: %s\n", cmd);
+#ifdef WIN32
+	/* Are we printing "command:" after its output? */
+	if (mainThreadId != GetCurrentThreadId())
+		fprintf(log, "\n\n");
+#endif
 
 	/*
 	 * In Windows, we must close the log file at this point so the file is not
@@ -98,14 +129,18 @@ exec_prog(const char *log_file, const char *opt_log_file,
 	 */
 	fclose(log);
 
-	result = system(cmd);
-
-	umask(old_umask);
+#ifdef WIN32
+	/* see comment above */
+	if (mainThreadId == GetCurrentThreadId())
+#endif
+		result = system(cmd);
 
 	if (result != 0)
 	{
-		report_status(PG_REPORT, "*failure*");
+		/* we might be in on a progress status line, so go to the next line */
+		report_status(PG_REPORT, "\n*failure*");
 		fflush(stdout);
+
 		pg_log(PG_VERBOSE, "There were problems executing \"%s\"\n", cmd);
 		if (opt_log_file)
 			pg_log(throw_error ? PG_FATAL : PG_REPORT,
@@ -120,14 +155,14 @@ exec_prog(const char *log_file, const char *opt_log_file,
 	}
 
 #ifndef WIN32
-	/* 
-	 *	We can't do this on Windows because it will keep the "pg_ctl start"
-	 *	output filename open until the server stops, so we do the \n\n above
-	 *	on that platform.  We use a unique filename for "pg_ctl start" that is
-	 *	never reused while the server is running, so it works fine.  We could
-	 *	log these commands to a third file, but that just adds complexity.
+	/*
+	 * We can't do this on Windows because it will keep the "pg_ctl start"
+	 * output filename open until the server stops, so we do the \n\n above on
+	 * that platform.  We use a unique filename for "pg_ctl start" that is
+	 * never reused while the server is running, so it works fine.  We could
+	 * log these commands to a third file, but that just adds complexity.
 	 */
-	if ((log = fopen_priv(log_file, "a")) == NULL)
+	if ((log = fopen(log_file, "a")) == NULL)
 		pg_log(PG_FATAL, "cannot write to log file %s\n", log_file);
 	fprintf(log, "\n\n");
 	fclose(log);
@@ -138,13 +173,12 @@ exec_prog(const char *log_file, const char *opt_log_file,
 
 
 /*
- * is_server_running()
+ * pid_lock_file_exists()
  *
- * checks whether postmaster on the given data directory is running or not.
- * The check is performed by looking for the existence of postmaster.pid file.
+ * Checks whether the postmaster.pid file exists.
  */
 bool
-is_server_running(const char *datadir)
+pid_lock_file_exists(const char *datadir)
 {
 	char		path[MAXPGPATH];
 	int			fd;
@@ -177,9 +211,6 @@ is_server_running(const char *datadir)
 void
 verify_directories(void)
 {
-
-	prep_status("Checking current, bin, and data directories");
-
 #ifndef WIN32
 	if (access(".", R_OK | W_OK | X_OK) != 0)
 #else
@@ -192,7 +223,6 @@ verify_directories(void)
 	check_data_dir(old_cluster.pgdata);
 	check_bin_dir(&new_cluster);
 	check_data_dir(new_cluster.pgdata);
-	check_ok();
 }
 
 
@@ -210,7 +240,7 @@ win32_check_directory_write_permissions(void)
 	int			fd;
 
 	/*
-	 * We open a file we would normally create anyway.	We do this even in
+	 * We open a file we would normally create anyway.  We do this even in
 	 * 'check' mode, which isn't ideal, but this is the best we can do.
 	 */
 	if ((fd = open(GLOBALS_DUMP_FILE, O_RDWR | O_CREAT, S_IRUSR | S_IWUSR)) < 0)
@@ -227,7 +257,7 @@ win32_check_directory_write_permissions(void)
  *
  *	This function validates the given cluster directory - we search for a
  *	small set of subdirectories that we expect to find in a valid $PGDATA
- *	directory.	If any of the subdirectories are missing (or secured against
+ *	directory.  If any of the subdirectories are missing (or secured against
  *	us) we display an error message and exit()
  *
  */
@@ -267,7 +297,7 @@ check_data_dir(const char *pg_data)
  * check_bin_dir()
  *
  *	This function searches for the executables that we expect to find
- *	in the binaries directory.	If we find that a required executable
+ *	in the binaries directory.  If we find that a required executable
  *	is missing (or secured against us), we display an error message and
  *	exit().
  */
@@ -290,7 +320,6 @@ check_bin_dir(ClusterInfo *cluster)
 	if (cluster == &new_cluster)
 	{
 		/* these are only needed in the new cluster */
-		validate_exec(cluster->bindir, "pg_config");
 		validate_exec(cluster->bindir, "psql");
 		validate_exec(cluster->bindir, "pg_dumpall");
 	}
