@@ -4,7 +4,7 @@
  *	  implementation for PostgreSQL generic linked list package
  *
  *
- * Portions Copyright (c) 1996-2011, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -15,6 +15,9 @@
  */
 #include "postgres.h"
 
+/* see pg_list.h */
+#define PG_LIST_INCLUDE_DEFINITIONS
+
 #include "nodes/pg_list.h"
 
 
@@ -31,7 +34,7 @@
  * Check that the specified List is valid (so far as we can tell).
  */
 static void
-check_list_invariants(List *list)
+check_list_invariants(const List *list)
 {
 	if (list == NIL)
 		return;
@@ -383,7 +386,7 @@ list_truncate(List *list, int new_size)
  * failure if there is no such cell.
  */
 static ListCell *
-list_nth_cell(List *list, int n)
+list_nth_cell(const List *list, int n)
 {
 	ListCell   *match;
 
@@ -407,7 +410,7 @@ list_nth_cell(List *list, int n)
  * specified list. (List elements begin at 0.)
  */
 void *
-list_nth(List *list, int n)
+list_nth(const List *list, int n)
 {
 	Assert(IsPointerList(list));
 	return lfirst(list_nth_cell(list, n));
@@ -418,7 +421,7 @@ list_nth(List *list, int n)
  * specified list.
  */
 int
-list_nth_int(List *list, int n)
+list_nth_int(const List *list, int n)
 {
 	Assert(IsIntegerList(list));
 	return lfirst_int(list_nth_cell(list, n));
@@ -429,7 +432,7 @@ list_nth_int(List *list, int n)
  * list.
  */
 Oid
-list_nth_oid(List *list, int n)
+list_nth_oid(const List *list, int n)
 {
 	Assert(IsOidList(list));
 	return lfirst_oid(list_nth_cell(list, n));
@@ -441,9 +444,9 @@ list_nth_oid(List *list, int n)
  * Node as 'datum'.
  */
 bool
-list_member(List *list, void *datum)
+list_member(const List *list, const void *datum)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsPointerList(list));
 	check_list_invariants(list);
@@ -462,9 +465,9 @@ list_member(List *list, void *datum)
  * determined by using simple pointer comparison.
  */
 bool
-list_member_ptr(List *list, void *datum)
+list_member_ptr(const List *list, const void *datum)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsPointerList(list));
 	check_list_invariants(list);
@@ -482,9 +485,9 @@ list_member_ptr(List *list, void *datum)
  * Return true iff the integer 'datum' is a member of the list.
  */
 bool
-list_member_int(List *list, int datum)
+list_member_int(const List *list, int datum)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsIntegerList(list));
 	check_list_invariants(list);
@@ -502,9 +505,9 @@ list_member_int(List *list, int datum)
  * Return true iff the OID 'datum' is a member of the list.
  */
 bool
-list_member_oid(List *list, Oid datum)
+list_member_oid(const List *list, Oid datum)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsOidList(list));
 	check_list_invariants(list);
@@ -694,10 +697,10 @@ list_delete_first(List *list)
  * performance bottleneck.
  */
 List *
-list_union(List *list1, List *list2)
+list_union(const List *list1, const List *list2)
 {
 	List	   *result;
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsPointerList(list1));
 	Assert(IsPointerList(list2));
@@ -718,10 +721,10 @@ list_union(List *list1, List *list2)
  * pointer comparison.
  */
 List *
-list_union_ptr(List *list1, List *list2)
+list_union_ptr(const List *list1, const List *list2)
 {
 	List	   *result;
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsPointerList(list1));
 	Assert(IsPointerList(list2));
@@ -741,10 +744,10 @@ list_union_ptr(List *list1, List *list2)
  * This variant of list_union() operates upon lists of integers.
  */
 List *
-list_union_int(List *list1, List *list2)
+list_union_int(const List *list1, const List *list2)
 {
 	List	   *result;
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsIntegerList(list1));
 	Assert(IsIntegerList(list2));
@@ -764,10 +767,10 @@ list_union_int(List *list1, List *list2)
  * This variant of list_union() operates upon lists of OIDs.
  */
 List *
-list_union_oid(List *list1, List *list2)
+list_union_oid(const List *list1, const List *list2)
 {
 	List	   *result;
-	ListCell   *cell;
+	const ListCell *cell;
 
 	Assert(IsOidList(list1));
 	Assert(IsOidList(list2));
@@ -797,10 +800,10 @@ list_union_oid(List *list1, List *list2)
  * to in the result.
  */
 List *
-list_intersection(List *list1, List *list2)
+list_intersection(const List *list1, const List *list2)
 {
 	List	   *result;
-	ListCell   *cell;
+	const ListCell *cell;
 
 	if (list1 == NIL || list2 == NIL)
 		return NIL;
@@ -829,9 +832,9 @@ list_intersection(List *list1, List *list2)
  * membership via equal()
  */
 List *
-list_difference(List *list1, List *list2)
+list_difference(const List *list1, const List *list2)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 	List	   *result = NIL;
 
 	Assert(IsPointerList(list1));
@@ -855,9 +858,9 @@ list_difference(List *list1, List *list2)
  * simple pointer equality.
  */
 List *
-list_difference_ptr(List *list1, List *list2)
+list_difference_ptr(const List *list1, const List *list2)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 	List	   *result = NIL;
 
 	Assert(IsPointerList(list1));
@@ -880,9 +883,9 @@ list_difference_ptr(List *list1, List *list2)
  * This variant of list_difference() operates upon lists of integers.
  */
 List *
-list_difference_int(List *list1, List *list2)
+list_difference_int(const List *list1, const List *list2)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 	List	   *result = NIL;
 
 	Assert(IsIntegerList(list1));
@@ -905,9 +908,9 @@ list_difference_int(List *list1, List *list2)
  * This variant of list_difference() operates upon lists of OIDs.
  */
 List *
-list_difference_oid(List *list1, List *list2)
+list_difference_oid(const List *list1, const List *list2)
 {
-	ListCell   *cell;
+	const ListCell *cell;
 	List	   *result = NIL;
 
 	Assert(IsOidList(list1));
@@ -1131,7 +1134,7 @@ list_free_deep(List *list)
  * Return a shallow copy of the specified list.
  */
 List *
-list_copy(List *oldlist)
+list_copy(const List *oldlist)
 {
 	List	   *newlist;
 	ListCell   *newlist_prev;
@@ -1174,7 +1177,7 @@ list_copy(List *oldlist)
  * Return a shallow copy of the specified list, without the first N elements.
  */
 List *
-list_copy_tail(List *oldlist, int nskip)
+list_copy_tail(const List *oldlist, int nskip)
 {
 	List	   *newlist;
 	ListCell   *newlist_prev;
@@ -1223,31 +1226,6 @@ list_copy_tail(List *oldlist, int nskip)
 	return newlist;
 }
 
-/*
- * pg_list.h defines inline versions of these functions if allowed by the
- * compiler; in which case the definitions below are skipped.
- */
-#ifndef USE_INLINE
-
-ListCell *
-list_head(List *l)
-{
-	return l ? l->head : NULL;
-}
-
-ListCell *
-list_tail(List *l)
-{
-	return l ? l->tail : NULL;
-}
-
-int
-list_length(List *l)
-{
-	return l ? l->length : 0;
-}
-#endif   /* ! USE_INLINE */
-
 /*
  * Temporary compatibility functions
  *
@@ -1264,10 +1242,10 @@ list_length(List *l)
  * list_length() macro in order to avoid the overhead of a function
  * call.
  */
-int			length(List *list);
+int			length(const List *list);
 
 int
-length(List *list)
+length(const List *list)
 {
 	return list_length(list);
 }
