@@ -20,7 +20,7 @@
  * tree after local transformations that might introduce nested AND/ORs.
  *
  *
- * Portions Copyright (c) 1996-2012, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
@@ -40,7 +40,7 @@
 
 static List *pull_ands(List *andlist);
 static List *pull_ors(List *orlist);
-static Expr *find_duplicate_ors(Expr *qual);
+static Expr *find_duplicate_ors(Expr *qual, bool is_check);
 static Expr *process_duplicate_ors(List *orlist);
 
 
@@ -54,12 +54,12 @@ static Expr *process_duplicate_ors(List *orlist);
  * Although this can be invoked on its own, it's mainly intended as a helper
  * for eval_const_expressions(), and that context drives several design
  * decisions.  In particular, if the input is already AND/OR flat, we must
- * preserve that property.	We also don't bother to recurse in situations
+ * preserve that property.  We also don't bother to recurse in situations
  * where we can assume that lower-level executions of eval_const_expressions
  * would already have simplified sub-clauses of the input.
  *
  * The difference between this and a simple make_notclause() is that this
- * tries to get rid of the NOT node by logical simplification.	It's clearly
+ * tries to get rid of the NOT node by logical simplification.  It's clearly
  * always a win if the NOT node can be eliminated altogether.  However, our
  * use of DeMorgan's laws could result in having more NOT nodes rather than
  * fewer.  We do that unconditionally anyway, because in WHERE clauses it's
@@ -152,7 +152,7 @@ negate_clause(Node *node)
 						 * those properties.  For example, if no direct child of
 						 * the given AND clause is an AND or a NOT-above-OR, then
 						 * the recursive calls of negate_clause() can't return any
-						 * OR clauses.	So we needn't call pull_ors() before
+						 * OR clauses.  So we needn't call pull_ors() before
 						 * building a new OR clause.  Similarly for the OR case.
 						 *--------------------
 						 */
@@ -268,6 +268,24 @@ negate_clause(Node *node)
  * canonicalize_qual
  *	  Convert a qualification expression to the most useful form.
  *
+ * Backwards-compatibility wrapper for use by external code that hasn't
+ * been updated.
+ */
+Expr *
+canonicalize_qual(Expr *qual)
+{
+	return canonicalize_qual_ext(qual, false);
+}
+
+/*
+ * canonicalize_qual_ext
+ *	  Convert a qualification expression to the most useful form.
+ *
+ * This is primarily intended to be used on top-level WHERE (or JOIN/ON)
+ * clauses.  It can also be used on top-level CHECK constraints, for which
+ * pass is_check = true.  DO NOT call it on any expression that is not known
+ * to be one or the other, as it might apply inappropriate simplifications.
+ *
  * The name of this routine is a holdover from a time when it would try to
  * force the expression into canonical AND-of-ORs or OR-of-ANDs form.
  * Eventually, we recognized that that had more theoretical purity than
@@ -282,7 +300,7 @@ negate_clause(Node *node)
  * Returns the modified qualification.
  */
 Expr *
-canonicalize_qual(Expr *qual)
+canonicalize_qual_ext(Expr *qual, bool is_check)
 {
 	Expr	   *newqual;
 
@@ -293,9 +311,9 @@ canonicalize_qual(Expr *qual)
 	/*
 	 * Pull up redundant subclauses in OR-of-AND trees.  We do this only
 	 * within the top-level AND/OR structure; there's no point in looking
-	 * deeper.
+	 * deeper.  Also remove any NULL constants in the top-level structure.
 	 */
-	newqual = find_duplicate_ors(qual);
+	newqual = find_duplicate_ors(qual, is_check);
 
 	return newqual;
 }
@@ -374,7 +392,7 @@ pull_ors(List *orlist)
  *
  * This may seem like a fairly useless activity, but it turns out to be
  * applicable to many machine-generated queries, and there are also queries
- * in some of the TPC benchmarks that need it.	This was in fact almost the
+ * in some of the TPC benchmarks that need it.  This was in fact almost the
  * sole useful side-effect of the old prepqual code that tried to force
  * the query into canonical AND-of-ORs form: the canonical equivalent of
  *		((A AND B) OR (A AND C))
@@ -393,10 +411,18 @@ pull_ors(List *orlist)
  *	  OR clauses to which the inverse OR distributive law might apply.
  *	  Only the top-level AND/OR structure is searched.
  *
- * Returns the modified qualification.	AND/OR flatness is preserved.
+ * While at it, we remove any NULL constants within the top-level AND/OR
+ * structure, eg in a WHERE clause, "x OR NULL::boolean" is reduced to "x".
+ * In general that would change the result, so eval_const_expressions can't
+ * do it; but at top level of WHERE, we don't need to distinguish between
+ * FALSE and NULL results, so it's valid to treat NULL::boolean the same
+ * as FALSE and then simplify AND/OR accordingly.  Conversely, in a top-level
+ * CHECK constraint, we may treat a NULL the same as TRUE.
+ *
+ * Returns the modified qualification.  AND/OR flatness is preserved.
  */
 static Expr *
-find_duplicate_ors(Expr *qual)
+find_duplicate_ors(Expr *qual, bool is_check)
 {
 	if (or_clause((Node *) qual))
 	{
@@ -405,12 +431,41 @@ find_duplicate_ors(Expr *qual)
 
 		/* Recurse */
 		foreach(temp, ((BoolExpr *) qual)->args)
-			orlist = lappend(orlist, find_duplicate_ors(lfirst(temp)));
+		{
+			Expr	   *arg = (Expr *) lfirst(temp);
 
-		/*
-		 * Don't need pull_ors() since this routine will never introduce an OR
-		 * where there wasn't one before.
-		 */
+			arg = find_duplicate_ors(arg, is_check);
+
+			/* Get rid of any constant inputs */
+			if (arg && IsA(arg, Const))
+			{
+				Const	   *carg = (Const *) arg;
+
+				if (is_check)
+				{
+					/* Within OR in CHECK, drop constant FALSE */
+					if (!carg->constisnull && !DatumGetBool(carg->constvalue))
+						continue;
+					/* Constant TRUE or NULL, so OR reduces to TRUE */
+					return (Expr *) makeBoolConst(true, false);
+				}
+				else
+				{
+					/* Within OR in WHERE, drop constant FALSE or NULL */
+					if (carg->constisnull || !DatumGetBool(carg->constvalue))
+						continue;
+					/* Constant TRUE, so OR reduces to TRUE */
+					return arg;
+				}
+			}
+
+			orlist = lappend(orlist, arg);
+		}
+
+		/* Flatten any ORs pulled up to just below here */
+		orlist = pull_ors(orlist);
+
+		/* Now we can look for duplicate ORs */
 		return process_duplicate_ors(orlist);
 	}
 	else if (and_clause((Node *) qual))
@@ -420,10 +475,49 @@ find_duplicate_ors(Expr *qual)
 
 		/* Recurse */
 		foreach(temp, ((BoolExpr *) qual)->args)
-			andlist = lappend(andlist, find_duplicate_ors(lfirst(temp)));
+		{
+			Expr	   *arg = (Expr *) lfirst(temp);
+
+			arg = find_duplicate_ors(arg, is_check);
+
+			/* Get rid of any constant inputs */
+			if (arg && IsA(arg, Const))
+			{
+				Const	   *carg = (Const *) arg;
+
+				if (is_check)
+				{
+					/* Within AND in CHECK, drop constant TRUE or NULL */
+					if (carg->constisnull || DatumGetBool(carg->constvalue))
+						continue;
+					/* Constant FALSE, so AND reduces to FALSE */
+					return arg;
+				}
+				else
+				{
+					/* Within AND in WHERE, drop constant TRUE */
+					if (!carg->constisnull && DatumGetBool(carg->constvalue))
+						continue;
+					/* Constant FALSE or NULL, so AND reduces to FALSE */
+					return (Expr *) makeBoolConst(false, false);
+				}
+			}
+
+			andlist = lappend(andlist, arg);
+		}
+
 		/* Flatten any ANDs introduced just below here */
 		andlist = pull_ands(andlist);
-		/* The AND list can't get shorter, so result is always an AND */
+
+		/* AND of no inputs reduces to TRUE */
+		if (andlist == NIL)
+			return (Expr *) makeBoolConst(true, false);
+
+		/* Single-expression AND just reduces to that expression */
+		if (list_length(andlist) == 1)
+			return (Expr *) linitial(andlist);
+
+		/* Else we still need an AND node */
 		return make_andclause(andlist);
 	}
 	else
@@ -447,11 +541,13 @@ process_duplicate_ors(List *orlist)
 	List	   *neworlist;
 	ListCell   *temp;
 
+	/* OR of no inputs reduces to FALSE */
 	if (orlist == NIL)
-		return NULL;			/* probably can't happen */
-	if (list_length(orlist) == 1)		/* single-expression OR (can this
-										 * happen?) */
-		return linitial(orlist);
+		return (Expr *) makeBoolConst(false, false);
+
+	/* Single-expression OR just reduces to that expression */
+	if (list_length(orlist) == 1)
+		return (Expr *) linitial(orlist);
 
 	/*
 	 * Choose the shortest AND clause as the reference list --- obviously, any
