@@ -38,6 +38,8 @@ static RangeTblEntry *scanNameSpaceForRefname(ParseState *pstate,
 						const char *refname, int location);
 static RangeTblEntry *scanNameSpaceForRelid(ParseState *pstate, Oid relid,
 					  int location);
+static void check_lateral_ref_ok(ParseState *pstate, ParseNamespaceItem *nsitem,
+					 int location);
 static void markRTEForSelectPriv(ParseState *pstate, RangeTblEntry *rte,
 					 int rtindex, AttrNumber col);
 static void expandRelation(Oid relid, Alias *eref,
@@ -48,6 +50,7 @@ static void expandTupleDesc(TupleDesc tupdesc, Alias *eref,
 				int rtindex, int sublevels_up,
 				int location, bool include_dropped,
 				List **colnames, List **colvars);
+static int32 *getValuesTypmods(RangeTblEntry *rte);
 static int	specialAttNum(const char *attname);
 static bool isQueryUsingTempRelation_walker(Node *node, void *context);
 
@@ -69,7 +72,7 @@ static bool isQueryUsingTempRelation_walker(Node *node, void *context);
  *
  * A qualified refname (schemaname != NULL) can only match a relation RTE
  * that (a) has no alias and (b) is for the same relation identified by
- * schemaname.refname.	In this case we convert schemaname.refname to a
+ * schemaname.refname.  In this case we convert schemaname.refname to a
  * relation OID and search by relid, rather than by alias name.  This is
  * peculiar, but it's what SQL says to do.
  */
@@ -131,6 +134,18 @@ refnameRangeTblEntry(ParseState *pstate,
  * Search the query's table namespace for an RTE matching the
  * given unqualified refname.  Return the RTE if a unique match, or NULL
  * if no match.  Raise error if multiple matches.
+ *
+ * Note: it might seem that we shouldn't have to worry about the possibility
+ * of multiple matches; after all, the SQL standard disallows duplicate table
+ * aliases within a given SELECT level.  Historically, however, Postgres has
+ * been laxer than that.  For example, we allow
+ *		SELECT ... FROM tab1 x CROSS JOIN (tab2 x CROSS JOIN tab3 y) z
+ * on the grounds that the aliased join (z) hides the aliases within it,
+ * therefore there is no conflict between the two RTEs named "x".  However,
+ * if tab3 is a LATERAL subquery, then from within the subquery both "x"es
+ * are visible.  Rather than rejecting queries that used to work, we allow
+ * this situation, and complain only if there's actually an ambiguous
+ * reference to "x".
  */
 static RangeTblEntry *
 scanNameSpaceForRefname(ParseState *pstate, const char *refname, int location)
@@ -158,14 +173,7 @@ scanNameSpaceForRefname(ParseState *pstate, const char *refname, int location)
 						 errmsg("table reference \"%s\" is ambiguous",
 								refname),
 						 parser_errposition(pstate, location)));
-			/* SQL:2008 demands this be an error, not an invisible item */
-			if (nsitem->p_lateral_only && !nsitem->p_lateral_ok)
-				ereport(ERROR,
-						(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
-						 errmsg("invalid reference to FROM-clause entry for table \"%s\"",
-								refname),
-						 errdetail("The combining JOIN type must be INNER or LEFT for a LATERAL reference."),
-						 parser_errposition(pstate, location)));
+			check_lateral_ref_ok(pstate, nsitem, location);
 			result = rte;
 		}
 	}
@@ -174,9 +182,8 @@ scanNameSpaceForRefname(ParseState *pstate, const char *refname, int location)
 
 /*
  * Search the query's table namespace for a relation RTE matching the
- * given relation OID.	Return the RTE if a unique match, or NULL
- * if no match.  Raise error if multiple matches (which shouldn't
- * happen if the namespace was checked correctly when it was created).
+ * given relation OID.  Return the RTE if a unique match, or NULL
+ * if no match.  Raise error if multiple matches.
  *
  * See the comments for refnameRangeTblEntry to understand why this
  * acts the way it does.
@@ -210,14 +217,7 @@ scanNameSpaceForRelid(ParseState *pstate, Oid relid, int location)
 						 errmsg("table reference %u is ambiguous",
 								relid),
 						 parser_errposition(pstate, location)));
-			/* SQL:2008 demands this be an error, not an invisible item */
-			if (nsitem->p_lateral_only && !nsitem->p_lateral_ok)
-				ereport(ERROR,
-						(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
-						 errmsg("invalid reference to FROM-clause entry for table \"%s\"",
-								rte->eref->aliasname),
-						 errdetail("The combining JOIN type must be INNER or LEFT for a LATERAL reference."),
-						 parser_errposition(pstate, location)));
+			check_lateral_ref_ok(pstate, nsitem, location);
 			result = rte;
 		}
 	}
@@ -286,7 +286,7 @@ isFutureCTE(ParseState *pstate, const char *refname)
  *
  * This is different from refnameRangeTblEntry in that it considers every
  * entry in the ParseState's rangetable(s), not only those that are currently
- * visible in the p_namespace list(s).	This behavior is invalid per the SQL
+ * visible in the p_namespace list(s).  This behavior is invalid per the SQL
  * spec, and it may give ambiguous results (there might be multiple equally
  * valid matches, but only one will be returned).  This must be used ONLY
  * as a heuristic in giving suitable error messages.  See errorMissingRTE.
@@ -309,8 +309,8 @@ searchRangeTableForRel(ParseState *pstate, RangeVar *relation)
 	 * relation.
 	 *
 	 * NB: It's not critical that RangeVarGetRelid return the correct answer
-	 * here in the face of concurrent DDL.	If it doesn't, the worst case
-	 * scenario is a less-clear error message.	Also, the tables involved in
+	 * here in the face of concurrent DDL.  If it doesn't, the worst case
+	 * scenario is a less-clear error message.  Also, the tables involved in
 	 * the query are already locked, which reduces the number of cases in
 	 * which surprising behavior can occur.  So we do the name lookup
 	 * unlocked.
@@ -399,9 +399,40 @@ checkNameSpaceConflicts(ParseState *pstate, List *namespace1,
 	}
 }
 
+/*
+ * Complain if a namespace item is currently disallowed as a LATERAL reference.
+ * This enforces both SQL:2008's rather odd idea of what to do with a LATERAL
+ * reference to the wrong side of an outer join, and our own prohibition on
+ * referencing the target table of an UPDATE or DELETE as a lateral reference
+ * in a FROM/USING clause.
+ *
+ * Convenience subroutine to avoid multiple copies of a rather ugly ereport.
+ */
+static void
+check_lateral_ref_ok(ParseState *pstate, ParseNamespaceItem *nsitem,
+					 int location)
+{
+	if (nsitem->p_lateral_only && !nsitem->p_lateral_ok)
+	{
+		/* SQL:2008 demands this be an error, not an invisible item */
+		RangeTblEntry *rte = nsitem->p_rte;
+		char	   *refname = rte->eref->aliasname;
+
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
+			errmsg("invalid reference to FROM-clause entry for table \"%s\"",
+				   refname),
+				 (rte == pstate->p_target_rangetblentry) ?
+				 errhint("There is an entry for table \"%s\", but it cannot be referenced from this part of the query.",
+						 refname) :
+				 errdetail("The combining JOIN type must be INNER or LEFT for a LATERAL reference."),
+				 parser_errposition(pstate, location)));
+	}
+}
+
 /*
  * given an RTE, return RT index (starting with 1) of the entry,
- * and optionally get its nesting depth (0 = current).	If sublevels_up
+ * and optionally get its nesting depth (0 = current).  If sublevels_up
  * is NULL, only consider rels at the current nesting level.
  * Raises error if RTE not found.
  */
@@ -611,15 +642,8 @@ colNameToVar(ParseState *pstate, char *colname, bool localonly,
 							(errcode(ERRCODE_AMBIGUOUS_COLUMN),
 							 errmsg("column reference \"%s\" is ambiguous",
 									colname),
-							 parser_errposition(orig_pstate, location)));
-				/* SQL:2008 demands this be an error, not an invisible item */
-				if (nsitem->p_lateral_only && !nsitem->p_lateral_ok)
-					ereport(ERROR,
-							(errcode(ERRCODE_INVALID_COLUMN_REFERENCE),
-							 errmsg("invalid reference to FROM-clause entry for table \"%s\"",
-									rte->eref->aliasname),
-							 errdetail("The combining JOIN type must be INNER or LEFT for a LATERAL reference."),
-							 parser_errposition(orig_pstate, location)));
+							 parser_errposition(pstate, location)));
+				check_lateral_ref_ok(pstate, nsitem, location);
 				result = newresult;
 			}
 		}
@@ -640,7 +664,7 @@ colNameToVar(ParseState *pstate, char *colname, bool localonly,
  *
  * This is different from colNameToVar in that it considers every entry in
  * the ParseState's rangetable(s), not only those that are currently visible
- * in the p_namespace list(s).	This behavior is invalid per the SQL spec,
+ * in the p_namespace list(s).  This behavior is invalid per the SQL spec,
  * and it may give ambiguous results (there might be multiple equally valid
  * matches, but only one will be returned).  This must be used ONLY as a
  * heuristic in giving suitable error messages.  See errorMissingColumn.
@@ -991,7 +1015,7 @@ addRangeTableEntry(ParseState *pstate,
 
 	/*
 	 * Get the rel's OID.  This access also ensures that we have an up-to-date
-	 * relcache entry for the rel.	Since this is typically the first access
+	 * relcache entry for the rel.  Since this is typically the first access
 	 * to a rel in a statement, be careful to get the right access level
 	 * depending on whether we're doing SELECT FOR UPDATE/SHARE.
 	 */
@@ -1600,6 +1624,10 @@ isLockedRefname(ParseState *pstate, const char *refname)
  * and/or namespace list.  (We assume caller has checked for any
  * namespace conflicts.)  The RTE is always marked as unconditionally
  * visible, that is, not LATERAL-only.
+ *
+ * Note: some callers know that they can find the new ParseNamespaceItem
+ * at the end of the pstate->p_namespace list.  This is a bit ugly but not
+ * worth complicating this function's signature for.
  */
 void
 addRTEtoQuery(ParseState *pstate, RangeTblEntry *rte,
@@ -1681,13 +1709,22 @@ expandRTE(RangeTblEntry *rte, int rtindex, int sublevels_up,
 					varattno++;
 					Assert(varattno == te->resno);
 
+					/*
+					 * In scenarios where columns have been added to a view
+					 * since the outer query was originally parsed, there can
+					 * be more items in the subquery tlist than the outer
+					 * query expects.  We should ignore such extra column(s)
+					 * --- compare the behavior for composite-returning
+					 * functions, in the RTE_FUNCTION case below.
+					 */
+					if (!aliasp_item)
+						break;
+
 					if (colnames)
 					{
-						/* Assume there is one alias per target item */
 						char	   *label = strVal(lfirst(aliasp_item));
 
 						*colnames = lappend(*colnames, makeString(pstrdup(label)));
-						aliasp_item = lnext(aliasp_item);
 					}
 
 					if (colvars)
@@ -1703,6 +1740,8 @@ expandRTE(RangeTblEntry *rte, int rtindex, int sublevels_up,
 
 						*colvars = lappend(*colvars, varnode);
 					}
+
+					aliasp_item = lnext(aliasp_item);
 				}
 			}
 			break;
@@ -1787,9 +1826,22 @@ expandRTE(RangeTblEntry *rte, int rtindex, int sublevels_up,
 			{
 				/* Values RTE */
 				ListCell   *aliasp_item = list_head(rte->eref->colnames);
+				int32	   *coltypmods;
 				ListCell   *lcv;
 				ListCell   *lcc;
 
+				/*
+				 * It's okay to extract column types from the expressions in
+				 * the first row, since all rows will have been coerced to the
+				 * same types.  Their typmods might not be the same though, so
+				 * we potentially need to examine all rows to compute those.
+				 * Column collations are pre-computed in values_collations.
+				 */
+				if (colvars)
+					coltypmods = getValuesTypmods(rte);
+				else
+					coltypmods = NULL;
+
 				varattno = 0;
 				forboth(lcv, (List *) linitial(rte->values_lists),
 						lcc, rte->values_collations)
@@ -1814,13 +1866,15 @@ expandRTE(RangeTblEntry *rte, int rtindex, int sublevels_up,
 
 						varnode = makeVar(rtindex, varattno,
 										  exprType(col),
-										  exprTypmod(col),
+										  coltypmods[varattno - 1],
 										  colcollation,
 										  sublevels_up);
 						varnode->location = location;
 						*colvars = lappend(*colvars, varnode);
 					}
 				}
+				if (coltypmods)
+					pfree(coltypmods);
 			}
 			break;
 		case RTE_JOIN:
@@ -1926,6 +1980,8 @@ expandRTE(RangeTblEntry *rte, int rtindex, int sublevels_up,
 						varnode = makeVar(rtindex, varattno,
 										  coltype, coltypmod, colcoll,
 										  sublevels_up);
+						varnode->location = location;
+
 						*colvars = lappend(*colvars, varnode);
 					}
 				}
@@ -2016,6 +2072,74 @@ expandTupleDesc(TupleDesc tupdesc, Alias *eref,
 	}
 }
 
+/*
+ * getValuesTypmods -- expandRTE subroutine
+ *
+ * Identify per-column typmods for the given VALUES RTE.  Returns a
+ * palloc'd array.
+ */
+static int32 *
+getValuesTypmods(RangeTblEntry *rte)
+{
+	int32	   *coltypmods;
+	List	   *firstrow;
+	int			ncolumns,
+				nvalid,
+				i;
+	ListCell   *lc;
+
+	Assert(rte->values_lists != NIL);
+	firstrow = (List *) linitial(rte->values_lists);
+	ncolumns = list_length(firstrow);
+	coltypmods = (int32 *) palloc(ncolumns * sizeof(int32));
+	nvalid = 0;
+
+	/* Collect the typmods from the first VALUES row */
+	i = 0;
+	foreach(lc, firstrow)
+	{
+		Node	   *col = (Node *) lfirst(lc);
+
+		coltypmods[i] = exprTypmod(col);
+		if (coltypmods[i] >= 0)
+			nvalid++;
+		i++;
+	}
+
+	/*
+	 * Scan remaining rows; as soon as we have a non-matching typmod for a
+	 * column, reset that typmod to -1.  We can bail out early if all typmods
+	 * become -1.
+	 */
+	if (nvalid > 0)
+	{
+		for_each_cell(lc, lnext(list_head(rte->values_lists)))
+		{
+			List	   *thisrow = (List *) lfirst(lc);
+			ListCell   *lc2;
+
+			Assert(list_length(thisrow) == ncolumns);
+			i = 0;
+			foreach(lc2, thisrow)
+			{
+				Node	   *col = (Node *) lfirst(lc2);
+
+				if (coltypmods[i] >= 0 && coltypmods[i] != exprTypmod(col))
+				{
+					coltypmods[i] = -1;
+					nvalid--;
+				}
+				i++;
+			}
+
+			if (nvalid <= 0)
+				break;
+		}
+	}
+
+	return coltypmods;
+}
+
 /*
  * expandRelAttrs -
  *	  Workhorse for "*" expansion: produce a list of targetentries
@@ -2229,18 +2353,25 @@ get_rte_attribute_type(RangeTblEntry *rte, AttrNumber attnum,
 			break;
 		case RTE_VALUES:
 			{
-				/* Values RTE --- get type info from first sublist */
-				/* collation is stored separately, though */
+				/*
+				 * Values RTE --- we can get type info from first sublist, but
+				 * typmod may require scanning all sublists, and collation is
+				 * stored separately.  Using getValuesTypmods() is overkill,
+				 * but this path is taken so seldom for VALUES that it's not
+				 * worth writing extra code.
+				 */
 				List	   *collist = (List *) linitial(rte->values_lists);
 				Node	   *col;
+				int32	   *coltypmods = getValuesTypmods(rte);
 
 				if (attnum < 1 || attnum > list_length(collist))
 					elog(ERROR, "values list %s does not have attribute %d",
 						 rte->eref->aliasname, attnum);
 				col = (Node *) list_nth(collist, attnum - 1);
 				*vartype = exprType(col);
-				*vartypmod = exprTypmod(col);
+				*vartypmod = coltypmods[attnum - 1];
 				*varcollid = list_nth_oid(rte->values_collations, attnum - 1);
+				pfree(coltypmods);
 			}
 			break;
 		case RTE_JOIN:
@@ -2547,7 +2678,7 @@ errorMissingRTE(ParseState *pstate, RangeVar *relation)
 
 	/*
 	 * Check to see if there are any potential matches in the query's
-	 * rangetable.	(Note: cases involving a bad schema name in the RangeVar
+	 * rangetable.  (Note: cases involving a bad schema name in the RangeVar
 	 * will throw error immediately here.  That seems OK.)
 	 */
 	rte = searchRangeTableForRel(pstate, relation);
@@ -2601,7 +2732,7 @@ errorMissingColumn(ParseState *pstate,
 	RangeTblEntry *rte;
 
 	/*
-	 * If relname was given, just play dumb and report it.	(In practice, a
+	 * If relname was given, just play dumb and report it.  (In practice, a
 	 * bad qualification name should end up at errorMissingRTE, not here, so
 	 * no need to work hard on this case.)
 	 */
