@@ -3,12 +3,12 @@
  * dynloader.h
  *	  dynamic loader for HP-UX using the shared library mechanism
  *
- * Portions Copyright (c) 1996-2008, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/port/dynloader/hpux.h,v 1.14 2008/01/01 19:45:51 momjian Exp $
+ *	  src/backend/port/dynloader/hpux.h
  *
  *	NOTES
  *		all functions are defined here -- it's impossible to trace the
