@@ -33,6 +33,7 @@
 #include "access/hash.h"
 #include "lib/stringinfo.h"
 #include "libpq/pqformat.h"
+#include "miscadmin.h"
 #include "utils/builtins.h"
 #include "utils/date.h"
 #include "utils/int8.h"
@@ -89,6 +90,8 @@ range_in(PG_FUNCTION_ARGS)
 	RangeBound	lower;
 	RangeBound	upper;
 
+	check_stack_depth();		/* recurses when subtype is a range type */
+
 	cache = get_range_io_data(fcinfo, rngtypoid, IOFunc_input);
 
 	/* parse */
@@ -128,6 +131,8 @@ range_out(PG_FUNCTION_ARGS)
 	RangeBound	upper;
 	bool		empty;
 
+	check_stack_depth();		/* recurses when subtype is a range type */
+
 	cache = get_range_io_data(fcinfo, RangeTypeGetOid(range), IOFunc_output);
 
 	/* deserialize */
@@ -165,6 +170,8 @@ range_recv(PG_FUNCTION_ARGS)
 	RangeBound	lower;
 	RangeBound	upper;
 
+	check_stack_depth();		/* recurses when subtype is a range type */
+
 	cache = get_range_io_data(fcinfo, rngtypoid, IOFunc_receive);
 
 	/* receive the flags... */
@@ -245,6 +252,8 @@ range_send(PG_FUNCTION_ARGS)
 	RangeBound	upper;
 	bool		empty;
 
+	check_stack_depth();		/* recurses when subtype is a range type */
+
 	cache = get_range_io_data(fcinfo, RangeTypeGetOid(range), IOFunc_send);
 
 	/* deserialize */
@@ -917,12 +926,12 @@ range_overright_internal(TypeCacheEntry *typcache, RangeType *r1, RangeType *r2)
 
 	/* An empty range is neither before nor after any other range */
 	if (empty1 || empty2)
-		PG_RETURN_BOOL(false);
+		return false;
 
 	if (range_cmp_bounds(typcache, &lower1, &lower2) >= 0)
-		PG_RETURN_BOOL(true);
+		return true;
 
-	PG_RETURN_BOOL(false);
+	return false;
 }
 
 /* does not extend to left of? */
@@ -1114,6 +1123,8 @@ range_cmp(PG_FUNCTION_ARGS)
 				empty2;
 	int			cmp;
 
+	check_stack_depth();		/* recurses when subtype is a range type */
+
 	/* Different types should be prevented by ANYRANGE matching rules */
 	if (RangeTypeGetOid(r1) != RangeTypeGetOid(r2))
 		elog(ERROR, "range types do not match");
@@ -1125,16 +1136,22 @@ range_cmp(PG_FUNCTION_ARGS)
 
 	/* For b-tree use, empty ranges sort before all else */
 	if (empty1 && empty2)
-		PG_RETURN_INT32(0);
+		cmp = 0;
 	else if (empty1)
-		PG_RETURN_INT32(-1);
+		cmp = -1;
 	else if (empty2)
-		PG_RETURN_INT32(1);
+		cmp = 1;
+	else
+	{
+		cmp = range_cmp_bounds(typcache, &lower1, &lower2);
+		if (cmp == 0)
+			cmp = range_cmp_bounds(typcache, &upper1, &upper2);
+	}
 
-	if ((cmp = range_cmp_bounds(typcache, &lower1, &lower2)) != 0)
-		PG_RETURN_INT32(cmp);
+	PG_FREE_IF_COPY(r1, 0);
+	PG_FREE_IF_COPY(r2, 1);
 
-	PG_RETURN_INT32(range_cmp_bounds(typcache, &upper1, &upper2));
+	PG_RETURN_INT32(cmp);
 }
 
 /* inequality operators using the range_cmp function */
@@ -1187,6 +1204,8 @@ hash_range(PG_FUNCTION_ARGS)
 	uint32		lower_hash;
 	uint32		upper_hash;
 
+	check_stack_depth();		/* recurses when subtype is a range type */
+
 	typcache = range_get_typcache(fcinfo, RangeTypeGetOid(r));
 
 	/* deserialize */
@@ -1435,7 +1454,7 @@ tstzrange_subdiff(PG_FUNCTION_ARGS)
  *
  * This is for use by range-related functions that follow the convention
  * of using the fn_extra field as a pointer to the type cache entry for
- * the range type.	Functions that need to cache more information than
+ * the range type.  Functions that need to cache more information than
  * that must fend for themselves.
  */
 TypeCacheEntry *
@@ -1459,7 +1478,7 @@ range_get_typcache(FunctionCallInfo fcinfo, Oid rngtypid)
  * range_serialize: construct a range value from bounds and empty-flag
  *
  * This does not force canonicalization of the range value.  In most cases,
- * external callers should only be canonicalization functions.	Note that
+ * external callers should only be canonicalization functions.  Note that
  * we perform some datatype-independent canonicalization checks anyway.
  */
 RangeType *
@@ -1796,7 +1815,7 @@ range_cmp_bounds(TypeCacheEntry *typcache, RangeBound *b1, RangeBound *b2)
  * Compare two range boundary point values, returning <0, 0, or >0 according
  * to whether b1 is less than, equal to, or greater than b2.
  *
- * This is similar to but simpler than range_cmp_bounds().	We just compare
+ * This is similar to but simpler than range_cmp_bounds().  We just compare
  * the values held in b1 and b2, ignoring inclusive/exclusive flags.  The
  * lower/upper flags only matter for infinities, where they tell us if the
  * infinity is plus or minus.
@@ -2277,7 +2296,7 @@ range_contains_elem_internal(TypeCacheEntry *typcache, RangeType *r, Datum val)
 
 /*
  * datum_compute_size() and datum_write() are used to insert the bound
- * values into a range object.	They are modeled after heaptuple.c's
+ * values into a range object.  They are modeled after heaptuple.c's
  * heap_compute_data_size() and heap_fill_tuple(), but we need not handle
  * null values here.  TYPE_IS_PACKABLE must test the same conditions as
  * heaptuple.c's ATT_IS_PACKABLE macro.
