@@ -3,21 +3,23 @@
  * tsquery_util.c
  *	  Utilities for tsquery datatype
  *
- * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/adt/tsquery_util.c,v 1.13.6.1 2010/08/03 00:10:44 tgl Exp $
+ *	  src/backend/utils/adt/tsquery_util.c
  *
  *-------------------------------------------------------------------------
  */
 
 #include "postgres.h"
 
-#include "tsearch/ts_type.h"
 #include "tsearch/ts_utils.h"
 #include "miscadmin.h"
 
+/*
+ * Build QTNode tree for a tsquery given in QueryItem array format.
+ */
 QTNode *
 QT2QTN(QueryItem *in, char *operand)
 {
@@ -45,12 +47,18 @@ QT2QTN(QueryItem *in, char *operand)
 	else if (operand)
 	{
 		node->word = operand + in->qoperand.distance;
-		node->sign = 1 << (in->qoperand.valcrc % 32);
+		node->sign = ((uint32) 1) << (((unsigned int) in->qoperand.valcrc) % 32);
 	}
 
 	return node;
 }
 
+/*
+ * Free a QTNode tree.
+ *
+ * Referenced "word" and "valnode" items are freed if marked as transient
+ * by flags.
+ */
 void
 QTNFree(QTNode *in)
 {
@@ -63,26 +71,27 @@ QTNFree(QTNode *in)
 	if (in->valnode->type == QI_VAL && in->word && (in->flags & QTN_WORDFREE) != 0)
 		pfree(in->word);
 
-	if (in->child)
+	if (in->valnode->type == QI_OPR)
 	{
-		if (in->valnode)
-		{
-			if (in->valnode->type == QI_OPR && in->nchild > 0)
-			{
-				int			i;
+		int			i;
 
-				for (i = 0; i < in->nchild; i++)
-					QTNFree(in->child[i]);
-			}
-			if (in->flags & QTN_NEEDFREE)
-				pfree(in->valnode);
-		}
-		pfree(in->child);
+		for (i = 0; i < in->nchild; i++)
+			QTNFree(in->child[i]);
 	}
+	if (in->child)
+		pfree(in->child);
+
+	if (in->flags & QTN_NEEDFREE)
+		pfree(in->valnode);
 
 	pfree(in);
 }
 
+/*
+ * Sort comparator for QTNodes.
+ *
+ * The sort order is somewhat arbitrary.
+ */
 int
 QTNodeCompare(QTNode *an, QTNode *bn)
 {
@@ -132,12 +141,19 @@ QTNodeCompare(QTNode *an, QTNode *bn)
 	}
 }
 
+/*
+ * qsort comparator for QTNode pointers.
+ */
 static int
 cmpQTN(const void *a, const void *b)
 {
-	return QTNodeCompare(*(QTNode **) a, *(QTNode **) b);
+	return QTNodeCompare(*(QTNode *const *) a, *(QTNode *const *) b);
 }
 
+/*
+ * Canonicalize a QTNode tree by sorting the children of AND/OR nodes
+ * into an arbitrary but well-defined order.
+ */
 void
 QTNSort(QTNode *in)
 {
@@ -155,13 +171,16 @@ QTNSort(QTNode *in)
 		qsort((void *) in->child, in->nchild, sizeof(QTNode *), cmpQTN);
 }
 
+/*
+ * Are two QTNode trees equal according to QTNodeCompare?
+ */
 bool
 QTNEq(QTNode *a, QTNode *b)
 {
 	uint32		sign = a->sign & b->sign;
 
 	if (!(sign == a->sign && sign == b->sign))
-		return 0;
+		return false;
 
 	return (QTNodeCompare(a, b) == 0) ? true : false;
 }
@@ -187,11 +206,17 @@ QTNTernary(QTNode *in)
 	for (i = 0; i < in->nchild; i++)
 		QTNTernary(in->child[i]);
 
+	/* Only AND and OR are associative, so don't flatten other node types */
+	if (in->valnode->qoperator.oper != OP_AND &&
+		in->valnode->qoperator.oper != OP_OR)
+		return;
+
 	for (i = 0; i < in->nchild; i++)
 	{
 		QTNode	   *cc = in->child[i];
 
-		if (cc->valnode->type == QI_OPR && in->valnode->qoperator.oper == cc->valnode->qoperator.oper)
+		if (cc->valnode->type == QI_OPR &&
+			in->valnode->qoperator.oper == cc->valnode->qoperator.oper)
 		{
 			int			oldnchild = in->nchild;
 
@@ -230,9 +255,6 @@ QTNBinary(QTNode *in)
 	for (i = 0; i < in->nchild; i++)
 		QTNBinary(in->child[i]);
 
-	if (in->nchild <= 2)
-		return;
-
 	while (in->nchild > 2)
 	{
 		QTNode	   *nn = (QTNode *) palloc0(sizeof(QTNode));
@@ -257,8 +279,9 @@ QTNBinary(QTNode *in)
 }
 
 /*
- * Count the total length of operand string in tree, including '\0'-
- * terminators.
+ * Count the total length of operand strings in tree (including '\0'-
+ * terminators) and the total number of nodes.
+ * Caller must initialize *sumlen and *nnode to zeroes.
  */
 static void
 cntsize(QTNode *in, int *sumlen, int *nnode)
@@ -287,6 +310,10 @@ typedef struct
 	char	   *curoperand;
 } QTN2QTState;
 
+/*
+ * Recursively convert a QTNode tree into flat tsquery format.
+ * Caller must have allocated arrays of the correct size.
+ */
 static void
 fillQT(QTN2QTState *state, QTNode *in)
 {
@@ -324,6 +351,9 @@ fillQT(QTN2QTState *state, QTNode *in)
 	}
 }
 
+/*
+ * Build flat tsquery from a QTNode tree.
+ */
 TSQuery
 QTN2QT(QTNode *in)
 {
@@ -334,9 +364,14 @@ QTN2QT(QTNode *in)
 	QTN2QTState state;
 
 	cntsize(in, &sumlen, &nnode);
+
+	if (TSQUERY_TOO_BIG(nnode, sumlen))
+		ereport(ERROR,
+				(errcode(ERRCODE_PROGRAM_LIMIT_EXCEEDED),
+				 errmsg("tsquery is too large")));
 	len = COMPUTESIZE(nnode, sumlen);
 
-	out = (TSQuery) palloc(len);
+	out = (TSQuery) palloc0(len);
 	SET_VARSIZE(out, len);
 	out->size = nnode;
 
@@ -347,6 +382,11 @@ QTN2QT(QTNode *in)
 	return out;
 }
 
+/*
+ * Copy a QTNode tree.
+ *
+ * Modifiable copies of the words and valnodes are made, too.
+ */
 QTNode *
 QTNCopy(QTNode *in)
 {
@@ -382,6 +422,9 @@ QTNCopy(QTNode *in)
 	return out;
 }
 
+/*
+ * Clear the specified flag bit(s) in all nodes of a QTNode tree.
+ */
 void
 QTNClearFlags(QTNode *in, uint32 flags)
 {
