@@ -178,7 +178,7 @@ FindStreamingStart(uint32 *tli)
 		if (!ispartial)
 		{
 			struct stat statbuf;
-			char		fullpath[MAXPGPATH];
+			char		fullpath[MAXPGPATH * 2];
 
 			snprintf(fullpath, sizeof(fullpath), "%s/%s", basedir, dirent->d_name);
 			if (stat(fullpath, &statbuf) != 0)
@@ -377,7 +377,7 @@ main(int argc, char **argv)
 	int			option_index;
 
 	progname = get_progname(argv[0]);
-	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pg_receivexlog"));
+	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pg_basebackup"));
 
 	if (argc > 1)
 	{
