@@ -19,12 +19,14 @@
  *
  *
  * IDENTIFICATION
- *		$PostgreSQL: pgsql/src/bin/pg_dump/pg_backup_custom.c,v 1.47 2010/07/06 19:18:59 momjian Exp $
+ *		src/bin/pg_dump/pg_backup_custom.c
  *
  *-------------------------------------------------------------------------
  */
 
-#include "pg_backup_archiver.h"
+#include "compress_io.h"
+#include "parallel.h"
+#include "pg_backup_utils.h"
 
 /*--------
  * Routines in the format interface
@@ -58,20 +60,13 @@ static void _LoadBlobs(ArchiveHandle *AH, bool drop);
 static void _Clone(ArchiveHandle *AH);
 static void _DeClone(ArchiveHandle *AH);
 
-/*------------
- * Buffers used in zlib compression and extra data stored in archive and
- * in TOC entries.
- *------------
- */
-#define zlibOutSize 4096
-#define zlibInSize	4096
+static char *_MasterStartParallelItem(ArchiveHandle *AH, TocEntry *te, T_Action act);
+static int	_MasterEndParallelItem(ArchiveHandle *AH, TocEntry *te, const char *str, T_Action act);
+char	   *_WorkerJobRestoreCustom(ArchiveHandle *AH, TocEntry *te);
 
 typedef struct
 {
-	z_streamp	zp;
-	char	   *zlibOut;
-	char	   *zlibIn;
-	size_t		inSize;
+	CompressorState *cs;
 	int			hasSeek;
 	pgoff_t		filePos;
 	pgoff_t		dataStart;
@@ -89,11 +84,12 @@ typedef struct
  *------
  */
 static void _readBlockHeader(ArchiveHandle *AH, int *type, int *id);
-static void _StartDataCompressor(ArchiveHandle *AH, TocEntry *te);
-static void _EndDataCompressor(ArchiveHandle *AH, TocEntry *te);
 static pgoff_t _getFilePos(ArchiveHandle *AH, lclContext *ctx);
-static int	_DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush);
 
+static size_t _CustomWriteFunc(ArchiveHandle *AH, const char *buf, size_t len);
+static size_t _CustomReadFunc(ArchiveHandle *AH, char **buf, size_t *buflen);
+
+/* translator: this is a module name */
 static const char *modulename = gettext_noop("custom archiver");
 
 
@@ -136,39 +132,23 @@ InitArchiveFmt_Custom(ArchiveHandle *AH)
 	AH->ClonePtr = _Clone;
 	AH->DeClonePtr = _DeClone;
 
-	/*
-	 * Set up some special context used in compressing data.
-	 */
-	ctx = (lclContext *) calloc(1, sizeof(lclContext));
-	if (ctx == NULL)
-		die_horribly(AH, modulename, "out of memory\n");
-	AH->formatData = (void *) ctx;
+	AH->MasterStartParallelItemPtr = _MasterStartParallelItem;
+	AH->MasterEndParallelItemPtr = _MasterEndParallelItem;
 
-	ctx->zp = (z_streamp) malloc(sizeof(z_stream));
-	if (ctx->zp == NULL)
-		die_horribly(AH, modulename, "out of memory\n");
+	/* no parallel dump in the custom archive, only parallel restore */
+	AH->WorkerJobDumpPtr = NULL;
+	AH->WorkerJobRestorePtr = _WorkerJobRestoreCustom;
+
+	/* Set up a private area. */
+	ctx = (lclContext *) pg_malloc0(sizeof(lclContext));
+	AH->formatData = (void *) ctx;
 
 	/* Initialize LO buffering */
 	AH->lo_buf_size = LOBBUFSIZE;
-	AH->lo_buf = (void *) malloc(LOBBUFSIZE);
-	if (AH->lo_buf == NULL)
-		die_horribly(AH, modulename, "out of memory\n");
+	AH->lo_buf = (void *) pg_malloc(LOBBUFSIZE);
 
-	/*
-	 * zlibOutSize is the buffer size we tell zlib it can output to.  We
-	 * actually allocate one extra byte because some routines want to append a
-	 * trailing zero byte to the zlib output.  The input buffer is expansible
-	 * and is always of size ctx->inSize; zlibInSize is just the initial
-	 * default size for it.
-	 */
-	ctx->zlibOut = (char *) malloc(zlibOutSize + 1);
-	ctx->zlibIn = (char *) malloc(zlibInSize);
-	ctx->inSize = zlibInSize;
 	ctx->filePos = 0;
 
-	if (ctx->zlibOut == NULL || ctx->zlibIn == NULL)
-		die_horribly(AH, modulename, "out of memory\n");
-
 	/*
 	 * Now open the file
 	 */
@@ -178,15 +158,15 @@ InitArchiveFmt_Custom(ArchiveHandle *AH)
 		{
 			AH->FH = fopen(AH->fSpec, PG_BINARY_W);
 			if (!AH->FH)
-				die_horribly(AH, modulename, "could not open output file \"%s\": %s\n",
-							 AH->fSpec, strerror(errno));
+				exit_horribly(modulename, "could not open output file \"%s\": %s\n",
+							  AH->fSpec, strerror(errno));
 		}
 		else
 		{
 			AH->FH = stdout;
 			if (!AH->FH)
-				die_horribly(AH, modulename, "could not open output file: %s\n",
-							 strerror(errno));
+				exit_horribly(modulename, "could not open output file: %s\n",
+							  strerror(errno));
 		}
 
 		ctx->hasSeek = checkSeek(AH->FH);
@@ -197,15 +177,15 @@ InitArchiveFmt_Custom(ArchiveHandle *AH)
 		{
 			AH->FH = fopen(AH->fSpec, PG_BINARY_R);
 			if (!AH->FH)
-				die_horribly(AH, modulename, "could not open input file \"%s\": %s\n",
-							 AH->fSpec, strerror(errno));
+				exit_horribly(modulename, "could not open input file \"%s\": %s\n",
+							  AH->fSpec, strerror(errno));
 		}
 		else
 		{
 			AH->FH = stdin;
 			if (!AH->FH)
-				die_horribly(AH, modulename, "could not open input file: %s\n",
-							 strerror(errno));
+				exit_horribly(modulename, "could not open input file: %s\n",
+							  strerror(errno));
 		}
 
 		ctx->hasSeek = checkSeek(AH->FH);
@@ -222,14 +202,14 @@ InitArchiveFmt_Custom(ArchiveHandle *AH)
  *
  * Optional.
  *
- * Set up extrac format-related TOC data.
+ * Set up extract format-related TOC data.
 */
 static void
 _ArchiveEntry(ArchiveHandle *AH, TocEntry *te)
 {
 	lclTocEntry *ctx;
 
-	ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
+	ctx = (lclTocEntry *) pg_malloc0(sizeof(lclTocEntry));
 	if (te->dataDumper)
 		ctx->dataState = K_OFFSET_POS_NOT_SET;
 	else
@@ -260,18 +240,17 @@ _WriteExtraToc(ArchiveHandle *AH, TocEntry *te)
  *
  * Optional.
  *
- * Needs to match the order defined in _WriteExtraToc, and sould also
+ * Needs to match the order defined in _WriteExtraToc, and should also
  * use the Archiver input routines.
  */
 static void
 _ReadExtraToc(ArchiveHandle *AH, TocEntry *te)
 {
-	int			junk;
 	lclTocEntry *ctx = (lclTocEntry *) te->formatData;
 
 	if (ctx == NULL)
 	{
-		ctx = (lclTocEntry *) calloc(1, sizeof(lclTocEntry));
+		ctx = (lclTocEntry *) pg_malloc0(sizeof(lclTocEntry));
 		te->formatData = (void *) ctx;
 	}
 
@@ -282,7 +261,7 @@ _ReadExtraToc(ArchiveHandle *AH, TocEntry *te)
 	 * dump it at all.
 	 */
 	if (AH->version < K_VERS_1_7)
-		junk = ReadInt(AH);
+		ReadInt(AH);
 }
 
 /*
@@ -324,7 +303,7 @@ _StartData(ArchiveHandle *AH, TocEntry *te)
 	_WriteByte(AH, BLK_DATA);	/* Block type */
 	WriteInt(AH, te->dumpId);	/* For sanity check */
 
-	_StartDataCompressor(AH, te);
+	ctx->cs = AllocateCompressor(AH->compression, _CustomWriteFunc);
 }
 
 /*
@@ -340,17 +319,12 @@ static size_t
 _WriteData(ArchiveHandle *AH, const void *data, size_t dLen)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
-	z_streamp	zp = ctx->zp;
+	CompressorState *cs = ctx->cs;
 
-	zp->next_in = (void *) data;
-	zp->avail_in = dLen;
+	if (dLen == 0)
+		return 0;
 
-	while (zp->avail_in != 0)
-	{
-		/* printf("Deflating %lu bytes\n", (unsigned long) dLen); */
-		_DoDeflate(AH, ctx, 0);
-	}
-	return dLen;
+	return WriteDataToArchive(AH, cs, data, dLen);
 }
 
 /*
@@ -363,10 +337,11 @@ _WriteData(ArchiveHandle *AH, const void *data, size_t dLen)
 static void
 _EndData(ArchiveHandle *AH, TocEntry *te)
 {
-/*	lclContext *ctx = (lclContext *) AH->formatData; */
-/*	lclTocEntry *tctx = (lclTocEntry *) te->formatData; */
+	lclContext *ctx = (lclContext *) AH->formatData;
 
-	_EndDataCompressor(AH, te);
+	EndCompressor(AH, ctx->cs);
+	/* Send the end marker */
+	WriteInt(AH, 0);
 }
 
 /*
@@ -401,11 +376,14 @@ _StartBlobs(ArchiveHandle *AH, TocEntry *te)
 static void
 _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 {
+	lclContext *ctx = (lclContext *) AH->formatData;
+
 	if (oid == 0)
-		die_horribly(AH, modulename, "invalid OID for large object\n");
+		exit_horribly(modulename, "invalid OID for large object\n");
 
 	WriteInt(AH, oid);
-	_StartDataCompressor(AH, te);
+
+	ctx->cs = AllocateCompressor(AH->compression, _CustomWriteFunc);
 }
 
 /*
@@ -416,7 +394,11 @@ _StartBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 static void
 _EndBlob(ArchiveHandle *AH, TocEntry *te, Oid oid)
 {
-	_EndDataCompressor(AH, te);
+	lclContext *ctx = (lclContext *) AH->formatData;
+
+	EndCompressor(AH, ctx->cs);
+	/* Send the end marker */
+	WriteInt(AH, 0);
 }
 
 /*
@@ -467,9 +449,9 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 					break;
 
 				default:		/* Always have a default */
-					die_horribly(AH, modulename,
-								 "unrecognized data block type (%d) while searching archive\n",
-								 blkType);
+					exit_horribly(modulename,
+								  "unrecognized data block type (%d) while searching archive\n",
+								  blkType);
 					break;
 			}
 			_readBlockHeader(AH, &blkType, &id);
@@ -479,8 +461,8 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 	{
 		/* We can just seek to the place we need to be. */
 		if (fseeko(AH->FH, tctx->dataPos, SEEK_SET) != 0)
-			die_horribly(AH, modulename, "error during file seek: %s\n",
-						 strerror(errno));
+			exit_horribly(modulename, "error during file seek: %s\n",
+						  strerror(errno));
 
 		_readBlockHeader(AH, &blkType, &id);
 	}
@@ -489,25 +471,25 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 	if (blkType == EOF)
 	{
 		if (tctx->dataState == K_OFFSET_POS_NOT_SET)
-			die_horribly(AH, modulename, "could not find block ID %d in archive -- "
-						 "possibly due to out-of-order restore request, "
-						 "which cannot be handled due to lack of data offsets in archive\n",
-						 te->dumpId);
+			exit_horribly(modulename, "could not find block ID %d in archive -- "
+						  "possibly due to out-of-order restore request, "
+						  "which cannot be handled due to lack of data offsets in archive\n",
+						  te->dumpId);
 		else if (!ctx->hasSeek)
-			die_horribly(AH, modulename, "could not find block ID %d in archive -- "
-						 "possibly due to out-of-order restore request, "
+			exit_horribly(modulename, "could not find block ID %d in archive -- "
+						  "possibly due to out-of-order restore request, "
 				  "which cannot be handled due to non-seekable input file\n",
-						 te->dumpId);
+						  te->dumpId);
 		else	/* huh, the dataPos led us to EOF? */
-			die_horribly(AH, modulename, "could not find block ID %d in archive -- "
-						 "possibly corrupt archive\n",
-						 te->dumpId);
+			exit_horribly(modulename, "could not find block ID %d in archive -- "
+						  "possibly corrupt archive\n",
+						  te->dumpId);
 	}
 
 	/* Are we sane? */
 	if (id != te->dumpId)
-		die_horribly(AH, modulename, "found unexpected block ID (%d) when reading data -- expected %d\n",
-					 id, te->dumpId);
+		exit_horribly(modulename, "found unexpected block ID (%d) when reading data -- expected %d\n",
+					  id, te->dumpId);
 
 	switch (blkType)
 	{
@@ -520,8 +502,8 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 			break;
 
 		default:				/* Always have a default */
-			die_horribly(AH, modulename, "unrecognized data block type %d while restoring archive\n",
-						 blkType);
+			exit_horribly(modulename, "unrecognized data block type %d while restoring archive\n",
+						  blkType);
 			break;
 	}
 }
@@ -532,108 +514,7 @@ _PrintTocData(ArchiveHandle *AH, TocEntry *te, RestoreOptions *ropt)
 static void
 _PrintData(ArchiveHandle *AH)
 {
-	lclContext *ctx = (lclContext *) AH->formatData;
-	z_streamp	zp = ctx->zp;
-	size_t		blkLen;
-	char	   *in = ctx->zlibIn;
-	size_t		cnt;
-
-#ifdef HAVE_LIBZ
-	int			res;
-	char	   *out = ctx->zlibOut;
-#endif
-
-#ifdef HAVE_LIBZ
-
-	res = Z_OK;
-
-	if (AH->compression != 0)
-	{
-		zp->zalloc = Z_NULL;
-		zp->zfree = Z_NULL;
-		zp->opaque = Z_NULL;
-
-		if (inflateInit(zp) != Z_OK)
-			die_horribly(AH, modulename, "could not initialize compression library: %s\n", zp->msg);
-	}
-#endif
-
-	blkLen = ReadInt(AH);
-	while (blkLen != 0)
-	{
-		if (blkLen + 1 > ctx->inSize)
-		{
-			free(ctx->zlibIn);
-			ctx->zlibIn = NULL;
-			ctx->zlibIn = (char *) malloc(blkLen + 1);
-			if (!ctx->zlibIn)
-				die_horribly(AH, modulename, "out of memory\n");
-
-			ctx->inSize = blkLen + 1;
-			in = ctx->zlibIn;
-		}
-
-		cnt = fread(in, 1, blkLen, AH->FH);
-		if (cnt != blkLen)
-		{
-			if (feof(AH->FH))
-				die_horribly(AH, modulename,
-							 "could not read from input file: end of file\n");
-			else
-				die_horribly(AH, modulename,
-					"could not read from input file: %s\n", strerror(errno));
-		}
-
-		ctx->filePos += blkLen;
-
-		zp->next_in = (void *) in;
-		zp->avail_in = blkLen;
-
-#ifdef HAVE_LIBZ
-		if (AH->compression != 0)
-		{
-			while (zp->avail_in != 0)
-			{
-				zp->next_out = (void *) out;
-				zp->avail_out = zlibOutSize;
-				res = inflate(zp, 0);
-				if (res != Z_OK && res != Z_STREAM_END)
-					die_horribly(AH, modulename, "could not uncompress data: %s\n", zp->msg);
-
-				out[zlibOutSize - zp->avail_out] = '\0';
-				ahwrite(out, 1, zlibOutSize - zp->avail_out, AH);
-			}
-		}
-		else
-#endif
-		{
-			in[zp->avail_in] = '\0';
-			ahwrite(in, 1, zp->avail_in, AH);
-			zp->avail_in = 0;
-		}
-		blkLen = ReadInt(AH);
-	}
-
-#ifdef HAVE_LIBZ
-	if (AH->compression != 0)
-	{
-		zp->next_in = NULL;
-		zp->avail_in = 0;
-		while (res != Z_STREAM_END)
-		{
-			zp->next_out = (void *) out;
-			zp->avail_out = zlibOutSize;
-			res = inflate(zp, 0);
-			if (res != Z_OK && res != Z_STREAM_END)
-				die_horribly(AH, modulename, "could not uncompress data: %s\n", zp->msg);
-
-			out[zlibOutSize - zp->avail_out] = '\0';
-			ahwrite(out, 1, zlibOutSize - zp->avail_out, AH);
-		}
-		if (inflateEnd(zp) != Z_OK)
-			die_horribly(AH, modulename, "could not close compression library: %s\n", zp->msg);
-	}
-#endif
+	ReadDataFromArchive(AH, AH->compression, _CustomReadFunc);
 }
 
 static void
@@ -684,27 +565,28 @@ _skipData(ArchiveHandle *AH)
 {
 	lclContext *ctx = (lclContext *) AH->formatData;
 	size_t		blkLen;
-	char	   *in = ctx->zlibIn;
+	char	   *buf = NULL;
+	int			buflen = 0;
 	size_t		cnt;
 
 	blkLen = ReadInt(AH);
 	while (blkLen != 0)
 	{
-		if (blkLen > ctx->inSize)
+		if (blkLen > buflen)
 		{
-			free(ctx->zlibIn);
-			ctx->zlibIn = (char *) malloc(blkLen);
-			ctx->inSize = blkLen;
-			in = ctx->zlibIn;
+			if (buf)
+				free(buf);
+			buf = (char *) pg_malloc(blkLen);
+			buflen = blkLen;
 		}
-		cnt = fread(in, 1, blkLen, AH->FH);
+		cnt = fread(buf, 1, blkLen, AH->FH);
 		if (cnt != blkLen)
 		{
 			if (feof(AH->FH))
-				die_horribly(AH, modulename,
-							 "could not read from input file: end of file\n");
+				exit_horribly(modulename,
+							"could not read from input file: end of file\n");
 			else
-				die_horribly(AH, modulename,
+				exit_horribly(modulename,
 					"could not read from input file: %s\n", strerror(errno));
 		}
 
@@ -712,6 +594,9 @@ _skipData(ArchiveHandle *AH)
 
 		blkLen = ReadInt(AH);
 	}
+
+	if (buf)
+		free(buf);
 }
 
 /*
@@ -720,8 +605,6 @@ _skipData(ArchiveHandle *AH)
  * Mandatory.
  *
  * Called by the archiver to do integer & byte output to the archive.
- * These routines are only used to read & write headers & TOC.
- *
  */
 static int
 _WriteByte(ArchiveHandle *AH, const int i)
@@ -733,7 +616,7 @@ _WriteByte(ArchiveHandle *AH, const int i)
 	if (res != EOF)
 		ctx->filePos += 1;
 	else
-		die_horribly(AH, modulename, "could not write byte: %s\n", strerror(errno));
+		exit_horribly(modulename, "could not write byte: %s\n", strerror(errno));
 	return res;
 }
 
@@ -743,7 +626,6 @@ _WriteByte(ArchiveHandle *AH, const int i)
  * Mandatory
  *
  * Called by the archiver to read bytes & integers from the archive.
- * These routines are only used to read & write headers & TOC.
  * EOF should be treated as a fatal error.
  */
 static int
@@ -754,7 +636,7 @@ _ReadByte(ArchiveHandle *AH)
 
 	res = getc(AH->FH);
 	if (res == EOF)
-		die_horribly(AH, modulename, "unexpected end of file\n");
+		exit_horribly(modulename, "unexpected end of file\n");
 	ctx->filePos += 1;
 	return res;
 }
@@ -765,8 +647,6 @@ _ReadByte(ArchiveHandle *AH)
  * Mandatory.
  *
  * Called by the archiver to write a block of bytes to the archive.
- * These routines are only used to read & write headers & TOC.
- *
  */
 static size_t
 _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
@@ -777,8 +657,8 @@ _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
 	res = fwrite(buf, 1, len, AH->FH);
 
 	if (res != len)
-		die_horribly(AH, modulename,
-					 "could not write to output file: %s\n", strerror(errno));
+		exit_horribly(modulename,
+					"could not write to output file: %s\n", strerror(errno));
 
 	ctx->filePos += res;
 	return res;
@@ -790,8 +670,6 @@ _WriteBuf(ArchiveHandle *AH, const void *buf, size_t len)
  * Mandatory.
  *
  * Called by the archiver to read a block of bytes from the archive
- * These routines are only used to read & write headers & TOC.
- *
  */
 static size_t
 _ReadBuf(ArchiveHandle *AH, void *buf, size_t len)
@@ -832,7 +710,7 @@ _CloseArchive(ArchiveHandle *AH)
 		tpos = ftello(AH->FH);
 		WriteToc(AH);
 		ctx->dataStart = _getFilePos(AH, ctx);
-		WriteDataChunks(AH);
+		WriteDataChunks(AH, NULL);
 
 		/*
 		 * If possible, re-write the TOC in order to update the data offset
@@ -846,7 +724,7 @@ _CloseArchive(ArchiveHandle *AH)
 	}
 
 	if (fclose(AH->FH) != 0)
-		die_horribly(AH, modulename, "could not close archive file: %s\n", strerror(errno));
+		exit_horribly(modulename, "could not close archive file: %s\n", strerror(errno));
 
 	AH->FH = NULL;
 }
@@ -865,37 +743,143 @@ _ReopenArchive(ArchiveHandle *AH)
 	pgoff_t		tpos;
 
 	if (AH->mode == archModeWrite)
-		die_horribly(AH, modulename, "can only reopen input archives\n");
+		exit_horribly(modulename, "can only reopen input archives\n");
 
 	/*
 	 * These two cases are user-facing errors since they represent unsupported
 	 * (but not invalid) use-cases.  Word the error messages appropriately.
 	 */
 	if (AH->fSpec == NULL || strcmp(AH->fSpec, "") == 0)
-		die_horribly(AH, modulename, "parallel restore from stdin is not supported\n");
+		exit_horribly(modulename, "parallel restore from standard input is not supported\n");
 	if (!ctx->hasSeek)
-		die_horribly(AH, modulename, "parallel restore from non-seekable file is not supported\n");
+		exit_horribly(modulename, "parallel restore from non-seekable file is not supported\n");
 
 	errno = 0;
 	tpos = ftello(AH->FH);
 	if (errno)
-		die_horribly(AH, modulename, "could not determine seek position in archive file: %s\n",
-					 strerror(errno));
+		exit_horribly(modulename, "could not determine seek position in archive file: %s\n",
+					  strerror(errno));
 
 #ifndef WIN32
 	if (fclose(AH->FH) != 0)
-		die_horribly(AH, modulename, "could not close archive file: %s\n",
-					 strerror(errno));
+		exit_horribly(modulename, "could not close archive file: %s\n",
+					  strerror(errno));
 #endif
 
 	AH->FH = fopen(AH->fSpec, PG_BINARY_R);
 	if (!AH->FH)
-		die_horribly(AH, modulename, "could not open input file \"%s\": %s\n",
-					 AH->fSpec, strerror(errno));
+		exit_horribly(modulename, "could not open input file \"%s\": %s\n",
+					  AH->fSpec, strerror(errno));
 
 	if (fseeko(AH->FH, tpos, SEEK_SET) != 0)
-		die_horribly(AH, modulename, "could not set seek position in archive file: %s\n",
-					 strerror(errno));
+		exit_horribly(modulename, "could not set seek position in archive file: %s\n",
+					  strerror(errno));
+}
+
+/*
+ * Clone format-specific fields during parallel restoration.
+ */
+static void
+_Clone(ArchiveHandle *AH)
+{
+	lclContext *ctx = (lclContext *) AH->formatData;
+
+	AH->formatData = (lclContext *) pg_malloc(sizeof(lclContext));
+	memcpy(AH->formatData, ctx, sizeof(lclContext));
+	ctx = (lclContext *) AH->formatData;
+
+	/* sanity check, shouldn't happen */
+	if (ctx->cs != NULL)
+		exit_horribly(modulename, "compressor active\n");
+
+	/*
+	 * Note: we do not make a local lo_buf because we expect at most one BLOBS
+	 * entry per archive, so no parallelism is possible.  Likewise,
+	 * TOC-entry-local state isn't an issue because any one TOC entry is
+	 * touched by just one worker child.
+	 */
+}
+
+static void
+_DeClone(ArchiveHandle *AH)
+{
+	lclContext *ctx = (lclContext *) AH->formatData;
+
+	free(ctx);
+}
+
+/*
+ * This function is executed in the child of a parallel backup for the
+ * custom format archive and dumps the actual data.
+ */
+char *
+_WorkerJobRestoreCustom(ArchiveHandle *AH, TocEntry *te)
+{
+	/*
+	 * short fixed-size string + some ID so far, this needs to be malloc'ed
+	 * instead of static because we work with threads on windows
+	 */
+	const int	buflen = 64;
+	char	   *buf = (char *) pg_malloc(buflen);
+	ParallelArgs pargs;
+	int			status;
+
+	pargs.AH = AH;
+	pargs.te = te;
+
+	status = parallel_restore(&pargs);
+
+	snprintf(buf, buflen, "OK RESTORE %d %d %d", te->dumpId, status,
+			 status == WORKER_IGNORED_ERRORS ? AH->public.n_errors : 0);
+
+	return buf;
+}
+
+/*
+ * This function is executed in the parent process. Depending on the desired
+ * action (dump or restore) it creates a string that is understood by the
+ * _WorkerJobDump /_WorkerJobRestore functions of the dump format.
+ */
+static char *
+_MasterStartParallelItem(ArchiveHandle *AH, TocEntry *te, T_Action act)
+{
+	/*
+	 * A static char is okay here, even on Windows because we call this
+	 * function only from one process (the master).
+	 */
+	static char buf[64];		/* short fixed-size string + number */
+
+	/* no parallel dump in the custom archive format */
+	Assert(act == ACT_RESTORE);
+
+	snprintf(buf, sizeof(buf), "RESTORE %d", te->dumpId);
+
+	return buf;
+}
+
+/*
+ * This function is executed in the parent process. It analyzes the response of
+ * the _WorkerJobDump / _WorkerJobRestore functions of the dump format.
+ */
+static int
+_MasterEndParallelItem(ArchiveHandle *AH, TocEntry *te, const char *str, T_Action act)
+{
+	DumpId		dumpId;
+	int			nBytes,
+				status,
+				n_errors;
+
+	/* no parallel dump in the custom archive */
+	Assert(act == ACT_RESTORE);
+
+	sscanf(str, "%u %u %u%n", &dumpId, &status, &n_errors, &nBytes);
+
+	Assert(nBytes == strlen(str));
+	Assert(dumpId == te->dumpId);
+
+	AH->public.n_errors += n_errors;
+
+	return status;
 }
 
 /*--------------------------------------------------
@@ -942,7 +926,7 @@ _readBlockHeader(ArchiveHandle *AH, int *type, int *id)
 	int			byt;
 
 	/*
-	 * Note: if we are at EOF with a pre-1.3 input file, we'll die_horribly
+	 * Note: if we are at EOF with a pre-1.3 input file, we'll exit_horribly
 	 * inside ReadInt rather than returning EOF.  It doesn't seem worth
 	 * jumping through hoops to deal with that case better, because no such
 	 * files are likely to exist in the wild: only some 7.1 development
@@ -966,181 +950,52 @@ _readBlockHeader(ArchiveHandle *AH, int *type, int *id)
 }
 
 /*
- * If zlib is available, then startit up. This is called from
- * StartData & StartBlob. The buffers are setup in the Init routine.
+ * Callback function for WriteDataToArchive. Writes one block of (compressed)
+ * data to the archive.
  */
-static void
-_StartDataCompressor(ArchiveHandle *AH, TocEntry *te)
+static size_t
+_CustomWriteFunc(ArchiveHandle *AH, const char *buf, size_t len)
 {
-	lclContext *ctx = (lclContext *) AH->formatData;
-	z_streamp	zp = ctx->zp;
+	/* never write 0-byte blocks (this should not happen) */
+	if (len == 0)
+		return 0;
 
-#ifdef HAVE_LIBZ
-
-	if (AH->compression < 0 || AH->compression > 9)
-		AH->compression = Z_DEFAULT_COMPRESSION;
-
-	if (AH->compression != 0)
-	{
-		zp->zalloc = Z_NULL;
-		zp->zfree = Z_NULL;
-		zp->opaque = Z_NULL;
-
-		if (deflateInit(zp, AH->compression) != Z_OK)
-			die_horribly(AH, modulename, "could not initialize compression library: %s\n", zp->msg);
-	}
-#else
-
-	AH->compression = 0;
-#endif
-
-	/* Just be paranoid - maybe End is called after Start, with no Write */
-	zp->next_out = (void *) ctx->zlibOut;
-	zp->avail_out = zlibOutSize;
+	WriteInt(AH, len);
+	return _WriteBuf(AH, buf, len);
 }
 
 /*
- * Send compressed data to the output stream (via ahwrite).
- * Each data chunk is preceded by it's length.
- * In the case of Z0, or no zlib, just write the raw data.
- *
+ * Callback function for ReadDataFromArchive. To keep things simple, we
+ * always read one compressed block at a time.
  */
-static int
-_DoDeflate(ArchiveHandle *AH, lclContext *ctx, int flush)
+static size_t
+_CustomReadFunc(ArchiveHandle *AH, char **buf, size_t *buflen)
 {
-	z_streamp	zp = ctx->zp;
+	size_t		blkLen;
+	size_t		cnt;
 
-#ifdef HAVE_LIBZ
-	char	   *out = ctx->zlibOut;
-	int			res = Z_OK;
+	/* Read length */
+	blkLen = ReadInt(AH);
+	if (blkLen == 0)
+		return 0;
 
-	if (AH->compression != 0)
+	/* If the caller's buffer is not large enough, allocate a bigger one */
+	if (blkLen > *buflen)
 	{
-		res = deflate(zp, flush);
-		if (res == Z_STREAM_ERROR)
-			die_horribly(AH, modulename, "could not compress data: %s\n", zp->msg);
-
-		if (((flush == Z_FINISH) && (zp->avail_out < zlibOutSize))
-			|| (zp->avail_out == 0)
-			|| (zp->avail_in != 0)
-			)
-		{
-			/*
-			 * Extra paranoia: avoid zero-length chunks since a zero length
-			 * chunk is the EOF marker. This should never happen but...
-			 */
-			if (zp->avail_out < zlibOutSize)
-			{
-				/*
-				 * printf("Wrote %lu byte deflated chunk\n", (unsigned long)
-				 * (zlibOutSize - zp->avail_out));
-				 */
-				WriteInt(AH, zlibOutSize - zp->avail_out);
-				if (fwrite(out, 1, zlibOutSize - zp->avail_out, AH->FH) != (zlibOutSize - zp->avail_out))
-					die_horribly(AH, modulename, "could not write to output file: %s\n", strerror(errno));
-				ctx->filePos += zlibOutSize - zp->avail_out;
-			}
-			zp->next_out = (void *) out;
-			zp->avail_out = zlibOutSize;
-		}
+		free(*buf);
+		*buf = (char *) pg_malloc(blkLen);
+		*buflen = blkLen;
 	}
-	else
-#endif
+
+	cnt = _ReadBuf(AH, *buf, blkLen);
+	if (cnt != blkLen)
 	{
-		if (zp->avail_in > 0)
-		{
-			WriteInt(AH, zp->avail_in);
-			if (fwrite(zp->next_in, 1, zp->avail_in, AH->FH) != zp->avail_in)
-				die_horribly(AH, modulename, "could not write to output file: %s\n", strerror(errno));
-			ctx->filePos += zp->avail_in;
-			zp->avail_in = 0;
-		}
+		if (feof(AH->FH))
+			exit_horribly(modulename,
+						  "could not read from input file: end of file\n");
 		else
-		{
-#ifdef HAVE_LIBZ
-			if (flush == Z_FINISH)
-				res = Z_STREAM_END;
-#endif
-		}
+			exit_horribly(modulename,
+					"could not read from input file: %s\n", strerror(errno));
 	}
-
-#ifdef HAVE_LIBZ
-	return res;
-#else
-	return 1;
-#endif
-}
-
-/*
- * Terminate zlib context and flush it's buffers. If no zlib
- * then just return.
- */
-static void
-_EndDataCompressor(ArchiveHandle *AH, TocEntry *te)
-{
-
-#ifdef HAVE_LIBZ
-	lclContext *ctx = (lclContext *) AH->formatData;
-	z_streamp	zp = ctx->zp;
-	int			res;
-
-	if (AH->compression != 0)
-	{
-		zp->next_in = NULL;
-		zp->avail_in = 0;
-
-		do
-		{
-			/* printf("Ending data output\n"); */
-			res = _DoDeflate(AH, ctx, Z_FINISH);
-		} while (res != Z_STREAM_END);
-
-		if (deflateEnd(zp) != Z_OK)
-			die_horribly(AH, modulename, "could not close compression stream: %s\n", zp->msg);
-	}
-#endif
-
-	/* Send the end marker */
-	WriteInt(AH, 0);
-}
-
-
-/*
- * Clone format-specific fields during parallel restoration.
- */
-static void
-_Clone(ArchiveHandle *AH)
-{
-	lclContext *ctx = (lclContext *) AH->formatData;
-
-	AH->formatData = (lclContext *) malloc(sizeof(lclContext));
-	if (AH->formatData == NULL)
-		die_horribly(AH, modulename, "out of memory\n");
-	memcpy(AH->formatData, ctx, sizeof(lclContext));
-	ctx = (lclContext *) AH->formatData;
-
-	ctx->zp = (z_streamp) malloc(sizeof(z_stream));
-	ctx->zlibOut = (char *) malloc(zlibOutSize + 1);
-	ctx->zlibIn = (char *) malloc(ctx->inSize);
-
-	if (ctx->zp == NULL || ctx->zlibOut == NULL || ctx->zlibIn == NULL)
-		die_horribly(AH, modulename, "out of memory\n");
-
-	/*
-	 * Note: we do not make a local lo_buf because we expect at most one BLOBS
-	 * entry per archive, so no parallelism is possible.  Likewise,
-	 * TOC-entry-local state isn't an issue because any one TOC entry is
-	 * touched by just one worker child.
-	 */
-}
-
-static void
-_DeClone(ArchiveHandle *AH)
-{
-	lclContext *ctx = (lclContext *) AH->formatData;
-
-	free(ctx->zlibOut);
-	free(ctx->zlibIn);
-	free(ctx->zp);
-	free(ctx);
+	return cnt;
 }
