@@ -1,7 +1,7 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2011, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2013, PostgreSQL Global Development Group
  *
  * src/bin/psql/stringutils.h
  */
@@ -19,4 +19,9 @@ extern char *strtokx(const char *s,
 		bool del_quotes,
 		int encoding);
 
+extern void strip_quotes(char *source, char quote, char escape, int encoding);
+
+extern char *quote_if_needed(const char *source, const char *entails_quote,
+				char quote, char escape, int encoding);
+
 #endif   /* STRINGUTILS_H */
