@@ -2,20 +2,23 @@
  *	common.h
  *		Common support routines for bin/scripts/
  *
- *	Copyright (c) 2003-2008, PostgreSQL Global Development Group
+ *	Copyright (c) 2003-2013, PostgreSQL Global Development Group
  *
- *	$PostgreSQL: pgsql/src/bin/scripts/common.h,v 1.19 2008/01/01 19:45:56 momjian Exp $
+ *	src/bin/scripts/common.h
  */
 #ifndef COMMON_H
 #define COMMON_H
 
 #include "libpq-fe.h"
-#include "getopt_long.h"
-#include "pqexpbuffer.h"
+#include "getopt_long.h"		/* pgrminclude ignore */
+#include "pqexpbuffer.h"		/* pgrminclude ignore */
 
-#ifndef HAVE_INT_OPTRESET
-extern int	optreset;
-#endif
+enum trivalue
+{
+	TRI_DEFAULT,
+	TRI_NO,
+	TRI_YES
+};
 
 typedef void (*help_handler) (const char *progname);
 
@@ -27,7 +30,13 @@ extern void handle_help_version_opts(int argc, char *argv[],
 
 extern PGconn *connectDatabase(const char *dbname, const char *pghost,
 				const char *pgport, const char *pguser,
-				bool require_password, const char *progname);
+				enum trivalue prompt_password, const char *progname,
+				bool echo, bool fail_ok);
+
+extern PGconn *connectMaintenanceDatabase(const char *maintenance_db,
+						   const char *pghost, const char *pgport,
+						   const char *pguser, enum trivalue prompt_password,
+						   const char *progname, bool echo);
 
 extern PGresult *executeQuery(PGconn *conn, const char *query,
 			 const char *progname, bool echo);
@@ -38,6 +47,9 @@ extern void executeCommand(PGconn *conn, const char *query,
 extern bool executeMaintenanceCommand(PGconn *conn, const char *query,
 						  bool echo);
 
+extern void appendQualifiedRelation(PQExpBuffer buf, const char *name,
+						PGconn *conn, const char *progname, bool echo);
+
 extern bool yesno_prompt(const char *question);
 
 extern void setup_cancel_handler(void);
