@@ -2,10 +2,10 @@
  *
  * createuser
  *
- * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/bin/scripts/createuser.c,v 1.21 2005/10/15 02:49:41 momjian Exp $
+ * src/bin/scripts/createuser.c
  *
  *-------------------------------------------------------------------------
  */
@@ -17,7 +17,6 @@
 
 static void help(const char *progname);
 
-
 int
 main(int argc, char *argv[])
 {
@@ -25,9 +24,9 @@ main(int argc, char *argv[])
 		{"host", required_argument, NULL, 'h'},
 		{"port", required_argument, NULL, 'p'},
 		{"username", required_argument, NULL, 'U'},
+		{"no-password", no_argument, NULL, 'w'},
 		{"password", no_argument, NULL, 'W'},
 		{"echo", no_argument, NULL, 'e'},
-		{"quiet", no_argument, NULL, 'q'},
 		{"createdb", no_argument, NULL, 'd'},
 		{"no-createdb", no_argument, NULL, 'D'},
 		{"superuser", no_argument, NULL, 's'},
@@ -38,6 +37,9 @@ main(int argc, char *argv[])
 		{"no-inherit", no_argument, NULL, 'I'},
 		{"login", no_argument, NULL, 'l'},
 		{"no-login", no_argument, NULL, 'L'},
+		{"replication", no_argument, NULL, 1},
+		{"no-replication", no_argument, NULL, 2},
+		{"interactive", no_argument, NULL, 3},
 		/* adduser is obsolete, undocumented spelling of superuser */
 		{"adduser", no_argument, NULL, 'a'},
 		{"no-adduser", no_argument, NULL, 'A'},
@@ -51,100 +53,111 @@ main(int argc, char *argv[])
 	const char *progname;
 	int			optindex;
 	int			c;
-
-	char	   *newuser = NULL;
+	const char *newuser = NULL;
 	char	   *host = NULL;
 	char	   *port = NULL;
 	char	   *username = NULL;
-	bool		password = false;
+	enum trivalue prompt_password = TRI_DEFAULT;
 	bool		echo = false;
-	bool		quiet = false;
-	int			createdb = 0;
-	int			superuser = 0;
-	int			createrole = 0;
-	int			inherit = 0;
-	int			login = 0;
+	bool		interactive = false;
 	char	   *conn_limit = NULL;
 	bool		pwprompt = false;
-	int			encrypted = 0;	/* 0 uses server default */
 	char	   *newpassword = NULL;
 
+	/* Tri-valued variables.  */
+	enum trivalue createdb = TRI_DEFAULT,
+				superuser = TRI_DEFAULT,
+				createrole = TRI_DEFAULT,
+				inherit = TRI_DEFAULT,
+				login = TRI_DEFAULT,
+				replication = TRI_DEFAULT,
+				encrypted = TRI_DEFAULT;
+
 	PQExpBufferData sql;
 
 	PGconn	   *conn;
 	PGresult   *result;
 
 	progname = get_progname(argv[0]);
-	set_pglocale_pgservice(argv[0], "pgscripts");
+	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pgscripts"));
 
 	handle_help_version_opts(argc, argv, "createuser", help);
 
-	while ((c = getopt_long(argc, argv, "h:p:U:WeqdDsSaArRiIlLc:PEN",
+	while ((c = getopt_long(argc, argv, "h:p:U:wWedDsSaArRiIlLc:PEN",
 							long_options, &optindex)) != -1)
 	{
 		switch (c)
 		{
 			case 'h':
-				host = optarg;
+				host = pg_strdup(optarg);
 				break;
 			case 'p':
-				port = optarg;
+				port = pg_strdup(optarg);
 				break;
 			case 'U':
-				username = optarg;
+				username = pg_strdup(optarg);
+				break;
+			case 'w':
+				prompt_password = TRI_NO;
 				break;
 			case 'W':
-				password = true;
+				prompt_password = TRI_YES;
 				break;
 			case 'e':
 				echo = true;
 				break;
-			case 'q':
-				quiet = true;
-				break;
 			case 'd':
-				createdb = +1;
+				createdb = TRI_YES;
 				break;
 			case 'D':
-				createdb = -1;
+				createdb = TRI_NO;
 				break;
 			case 's':
 			case 'a':
-				superuser = +1;
+				superuser = TRI_YES;
 				break;
 			case 'S':
 			case 'A':
-				superuser = -1;
+				superuser = TRI_NO;
 				break;
 			case 'r':
-				createrole = +1;
+				createrole = TRI_YES;
 				break;
 			case 'R':
-				createrole = -1;
+				createrole = TRI_NO;
 				break;
 			case 'i':
-				inherit = +1;
+				inherit = TRI_YES;
 				break;
 			case 'I':
-				inherit = -1;
+				inherit = TRI_NO;
 				break;
 			case 'l':
-				login = +1;
+				login = TRI_YES;
 				break;
 			case 'L':
-				login = -1;
+				login = TRI_NO;
 				break;
 			case 'c':
-				conn_limit = optarg;
+				conn_limit = pg_strdup(optarg);
 				break;
 			case 'P':
 				pwprompt = true;
 				break;
 			case 'E':
-				encrypted = +1;
+				encrypted = TRI_YES;
 				break;
 			case 'N':
-				encrypted = -1;
+				encrypted = TRI_NO;
+				break;
+			case 1:
+				replication = TRI_YES;
+				break;
+			case 2:
+				replication = TRI_NO;
+				break;
+			case 3:
+				interactive = true;
 				break;
 			default:
 				fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
@@ -167,7 +180,17 @@ main(int argc, char *argv[])
 	}
 
 	if (newuser == NULL)
-		newuser = simple_prompt("Enter name of role to add: ", 128, true);
+	{
+		if (interactive)
+			newuser = simple_prompt("Enter name of role to add: ", 128, true);
+		else
+		{
+			if (getenv("PGUSER"))
+				newuser = getenv("PGUSER");
+			else
+				newuser = get_user_name(progname);
+		}
+	}
 
 	if (pwprompt)
 	{
@@ -187,94 +210,100 @@ main(int argc, char *argv[])
 
 	if (superuser == 0)
 	{
-		char	   *reply;
-
-		reply = simple_prompt("Shall the new role be a superuser? (y/n) ", 1, true);
-		if (check_yesno_response(reply) == 1)
-			superuser = +1;
+		if (interactive && yesno_prompt("Shall the new role be a superuser?"))
+			superuser = TRI_YES;
 		else
-			superuser = -1;
+			superuser = TRI_NO;
 	}
 
-	if (superuser == +1)
+	if (superuser == TRI_YES)
 	{
 		/* Not much point in trying to restrict a superuser */
-		createdb = +1;
-		createrole = +1;
+		createdb = TRI_YES;
+		createrole = TRI_YES;
 	}
 
 	if (createdb == 0)
 	{
-		char	   *reply;
-
-		reply = simple_prompt("Shall the new role be allowed to create databases? (y/n) ", 1, true);
-		if (check_yesno_response(reply) == 1)
-			createdb = +1;
+		if (interactive && yesno_prompt("Shall the new role be allowed to create databases?"))
+			createdb = TRI_YES;
 		else
-			createdb = -1;
+			createdb = TRI_NO;
 	}
 
 	if (createrole == 0)
 	{
-		char	   *reply;
-
-		reply = simple_prompt("Shall the new role be allowed to create more new roles? (y/n) ", 1, true);
-		if (check_yesno_response(reply) == 1)
-			createrole = +1;
+		if (interactive && yesno_prompt("Shall the new role be allowed to create more new roles?"))
+			createrole = TRI_YES;
 		else
-			createrole = -1;
+			createrole = TRI_NO;
 	}
 
 	if (inherit == 0)
-	{
-		/* silently default to YES */
-		inherit = +1;
-	}
+		inherit = TRI_YES;
 
 	if (login == 0)
-	{
-		/* silently default to YES */
-		login = +1;
-	}
+		login = TRI_YES;
+
+	conn = connectDatabase("postgres", host, port, username, prompt_password,
+						   progname, echo, false);
 
 	initPQExpBuffer(&sql);
 
 	printfPQExpBuffer(&sql, "CREATE ROLE %s", fmtId(newuser));
 	if (newpassword)
 	{
-		if (encrypted == +1)
+		if (encrypted == TRI_YES)
 			appendPQExpBuffer(&sql, " ENCRYPTED");
-		if (encrypted == -1)
+		if (encrypted == TRI_NO)
 			appendPQExpBuffer(&sql, " UNENCRYPTED");
 		appendPQExpBuffer(&sql, " PASSWORD ");
-		appendStringLiteral(&sql, newpassword, false);
+
+		if (encrypted != TRI_NO)
+		{
+			char	   *encrypted_password;
+
+			encrypted_password = PQencryptPassword(newpassword,
+												   newuser);
+			if (!encrypted_password)
+			{
+				fprintf(stderr, _("Password encryption failed.\n"));
+				exit(1);
+			}
+			appendStringLiteralConn(&sql, encrypted_password, conn);
+			PQfreemem(encrypted_password);
+		}
+		else
+			appendStringLiteralConn(&sql, newpassword, conn);
 	}
-	if (superuser == +1)
+	if (superuser == TRI_YES)
 		appendPQExpBuffer(&sql, " SUPERUSER");
-	if (superuser == -1)
+	if (superuser == TRI_NO)
 		appendPQExpBuffer(&sql, " NOSUPERUSER");
-	if (createdb == +1)
+	if (createdb == TRI_YES)
 		appendPQExpBuffer(&sql, " CREATEDB");
-	if (createdb == -1)
+	if (createdb == TRI_NO)
 		appendPQExpBuffer(&sql, " NOCREATEDB");
-	if (createrole == +1)
+	if (createrole == TRI_YES)
 		appendPQExpBuffer(&sql, " CREATEROLE");
-	if (createrole == -1)
+	if (createrole == TRI_NO)
 		appendPQExpBuffer(&sql, " NOCREATEROLE");
-	if (inherit == +1)
+	if (inherit == TRI_YES)
 		appendPQExpBuffer(&sql, " INHERIT");
-	if (inherit == -1)
+	if (inherit == TRI_NO)
 		appendPQExpBuffer(&sql, " NOINHERIT");
-	if (login == +1)
+	if (login == TRI_YES)
 		appendPQExpBuffer(&sql, " LOGIN");
-	if (login == -1)
+	if (login == TRI_NO)
 		appendPQExpBuffer(&sql, " NOLOGIN");
+	if (replication == TRI_YES)
+		appendPQExpBuffer(&sql, " REPLICATION");
+	if (replication == TRI_NO)
+		appendPQExpBuffer(&sql, " NOREPLICATION");
 	if (conn_limit != NULL)
 		appendPQExpBuffer(&sql, " CONNECTION LIMIT %s", conn_limit);
 	appendPQExpBuffer(&sql, ";\n");
 
-	conn = connectDatabase("postgres", host, port, username, password, progname);
-
 	if (echo)
 		printf("%s", sql.data);
 	result = PQexec(conn, sql.data);
@@ -287,12 +316,8 @@ main(int argc, char *argv[])
 		exit(1);
 	}
 
+	PQclear(result);
 	PQfinish(conn);
-	if (!quiet)
-	{
-		puts("CREATE ROLE");
-		fflush(stdout);
-	}
 	exit(0);
 }
 
@@ -304,31 +329,33 @@ help(const char *progname)
 	printf(_("Usage:\n"));
 	printf(_("  %s [OPTION]... [ROLENAME]\n"), progname);
 	printf(_("\nOptions:\n"));
-	printf(_("  -s, --superuser           role will be superuser\n"));
-	printf(_("  -S, --no-superuser        role will not be superuser\n"));
+	printf(_("  -c, --connection-limit=N  connection limit for role (default: no limit)\n"));
 	printf(_("  -d, --createdb            role can create new databases\n"));
-	printf(_("  -D, --no-createdb         role cannot create databases\n"));
-	printf(_("  -r, --createrole          role can create new roles\n"));
-	printf(_("  -R, --no-createrole       role cannot create roles\n"));
-	printf(_("  -l, --login               role can login (default)\n"));
-	printf(_("  -L, --no-login            role cannot login\n"));
+	printf(_("  -D, --no-createdb         role cannot create databases (default)\n"));
+	printf(_("  -e, --echo                show the commands being sent to the server\n"));
+	printf(_("  -E, --encrypted           encrypt stored password\n"));
 	printf(_("  -i, --inherit             role inherits privileges of roles it is a\n"
 			 "                            member of (default)\n"));
 	printf(_("  -I, --no-inherit          role does not inherit privileges\n"));
-	printf(_("  -c, --connection-limit=N  connection limit for role (default: no limit)\n"));
-	printf(_("  -P, --pwprompt            assign a password to new role\n"));
-	printf(_("  -E, --encrypted           encrypt stored password\n"));
+	printf(_("  -l, --login               role can login (default)\n"));
+	printf(_("  -L, --no-login            role cannot login\n"));
 	printf(_("  -N, --unencrypted         do not encrypt stored password\n"));
-	printf(_("  -e, --echo                show the commands being sent to the server\n"));
-	printf(_("  -q, --quiet               don't write any messages\n"));
-	printf(_("  --help                    show this help, then exit\n"));
-	printf(_("  --version                 output version information, then exit\n"));
+	printf(_("  -P, --pwprompt            assign a password to new role\n"));
+	printf(_("  -r, --createrole          role can create new roles\n"));
+	printf(_("  -R, --no-createrole       role cannot create roles (default)\n"));
+	printf(_("  -s, --superuser           role will be superuser\n"));
+	printf(_("  -S, --no-superuser        role will not be superuser (default)\n"));
+	printf(_("  -V, --version             output version information, then exit\n"));
+	printf(_("  --interactive             prompt for missing role name and attributes rather\n"
+			 "                            than using defaults\n"));
+	printf(_("  --replication             role can initiate replication\n"));
+	printf(_("  --no-replication          role cannot initiate replication\n"));
+	printf(_("  -?, --help                show this help, then exit\n"));
 	printf(_("\nConnection options:\n"));
 	printf(_("  -h, --host=HOSTNAME       database server host or socket directory\n"));
 	printf(_("  -p, --port=PORT           database server port\n"));
 	printf(_("  -U, --username=USERNAME   user name to connect as (not the one to create)\n"));
-	printf(_("  -W, --password            prompt for password to connect\n"));
-	printf(_("\nIf one of -s, -S, -d, -D, -r, -R and ROLENAME is not specified, you will\n"
-			 "be prompted interactively.\n"));
+	printf(_("  -w, --no-password         never prompt for password\n"));
+	printf(_("  -W, --password            force password prompt\n"));
 	printf(_("\nReport bugs to <pgsql-bugs@postgresql.org>.\n"));
 }
