@@ -2,24 +2,27 @@
  *
  * vacuumdb
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $PostgreSQL: pgsql/src/bin/scripts/vacuumdb.c,v 1.26.2.1 2009/10/16 10:38:55 heikki Exp $
+ * src/bin/scripts/vacuumdb.c
  *
  *-------------------------------------------------------------------------
  */
 
 #include "postgres_fe.h"
 #include "common.h"
+#include "dumputils.h"
 
 
-static void vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
-					bool freeze, const char *table,
-					const char *host, const char *port,
+static void vacuum_one_database(const char *dbname, bool full, bool verbose,
+					bool and_analyze, bool analyze_only, bool freeze,
+					const char *table, const char *host, const char *port,
 					const char *username, enum trivalue prompt_password,
 					const char *progname, bool echo);
-static void vacuum_all_databases(bool full, bool verbose, bool analyze, bool freeze,
+static void vacuum_all_databases(bool full, bool verbose, bool and_analyze,
+					 bool analyze_only, bool freeze,
+					 const char *maintenance_db,
 					 const char *host, const char *port,
 					 const char *username, enum trivalue prompt_password,
 					 const char *progname, bool echo, bool quiet);
@@ -40,11 +43,13 @@ main(int argc, char *argv[])
 		{"quiet", no_argument, NULL, 'q'},
 		{"dbname", required_argument, NULL, 'd'},
 		{"analyze", no_argument, NULL, 'z'},
+		{"analyze-only", no_argument, NULL, 'Z'},
 		{"freeze", no_argument, NULL, 'F'},
 		{"all", no_argument, NULL, 'a'},
 		{"table", required_argument, NULL, 't'},
 		{"full", no_argument, NULL, 'f'},
 		{"verbose", no_argument, NULL, 'v'},
+		{"maintenance-db", required_argument, NULL, 2},
 		{NULL, 0, NULL, 0}
 	};
 
@@ -53,36 +58,38 @@ main(int argc, char *argv[])
 	int			c;
 
 	const char *dbname = NULL;
+	const char *maintenance_db = NULL;
 	char	   *host = NULL;
 	char	   *port = NULL;
 	char	   *username = NULL;
 	enum trivalue prompt_password = TRI_DEFAULT;
 	bool		echo = false;
 	bool		quiet = false;
-	bool		analyze = false;
+	bool		and_analyze = false;
+	bool		analyze_only = false;
 	bool		freeze = false;
 	bool		alldb = false;
-	char	   *table = NULL;
 	bool		full = false;
 	bool		verbose = false;
+	SimpleStringList tables = {NULL, NULL};
 
 	progname = get_progname(argv[0]);
 	set_pglocale_pgservice(argv[0], PG_TEXTDOMAIN("pgscripts"));
 
 	handle_help_version_opts(argc, argv, "vacuumdb", help);
 
-	while ((c = getopt_long(argc, argv, "h:p:U:wWeqd:zaFt:fv", long_options, &optindex)) != -1)
+	while ((c = getopt_long(argc, argv, "h:p:U:wWeqd:zZFat:fv", long_options, &optindex)) != -1)
 	{
 		switch (c)
 		{
 			case 'h':
-				host = optarg;
+				host = pg_strdup(optarg);
 				break;
 			case 'p':
-				port = optarg;
+				port = pg_strdup(optarg);
 				break;
 			case 'U':
-				username = optarg;
+				username = pg_strdup(optarg);
 				break;
 			case 'w':
 				prompt_password = TRI_NO;
@@ -97,10 +104,13 @@ main(int argc, char *argv[])
 				quiet = true;
 				break;
 			case 'd':
-				dbname = optarg;
+				dbname = pg_strdup(optarg);
 				break;
 			case 'z':
-				analyze = true;
+				and_analyze = true;
+				break;
+			case 'Z':
+				analyze_only = true;
 				break;
 			case 'F':
 				freeze = true;
@@ -109,7 +119,7 @@ main(int argc, char *argv[])
 				alldb = true;
 				break;
 			case 't':
-				table = optarg;
+				simple_string_list_append(&tables, optarg);
 				break;
 			case 'f':
 				full = true;
@@ -117,24 +127,49 @@ main(int argc, char *argv[])
 			case 'v':
 				verbose = true;
 				break;
+			case 2:
+				maintenance_db = pg_strdup(optarg);
+				break;
 			default:
 				fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
 				exit(1);
 		}
 	}
 
-	switch (argc - optind)
+
+	/*
+	 * Non-option argument specifies database name as long as it wasn't
+	 * already specified with -d / --dbname
+	 */
+	if (optind < argc && dbname == NULL)
 	{
-		case 0:
-			break;
-		case 1:
-			dbname = argv[optind];
-			break;
-		default:
-			fprintf(stderr, _("%s: too many command-line arguments (first is \"%s\")\n"),
-					progname, argv[optind + 1]);
-			fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
+		dbname = argv[optind];
+		optind++;
+	}
+
+	if (optind < argc)
+	{
+		fprintf(stderr, _("%s: too many command-line arguments (first is \"%s\")\n"),
+				progname, argv[optind]);
+		fprintf(stderr, _("Try \"%s --help\" for more information.\n"), progname);
+		exit(1);
+	}
+
+	if (analyze_only)
+	{
+		if (full)
+		{
+			fprintf(stderr, _("%s: cannot use the \"full\" option when performing only analyze\n"),
+					progname);
 			exit(1);
+		}
+		if (freeze)
+		{
+			fprintf(stderr, _("%s: cannot use the \"freeze\" option when performing only analyze\n"),
+					progname);
+			exit(1);
+		}
+		/* allow 'and_analyze' with 'analyze_only' */
 	}
 
 	setup_cancel_handler();
@@ -147,16 +182,16 @@ main(int argc, char *argv[])
 					progname);
 			exit(1);
 		}
-		if (table)
+		if (tables.head != NULL)
 		{
-			fprintf(stderr, _("%s: cannot vacuum a specific table in all databases\n"),
+			fprintf(stderr, _("%s: cannot vacuum specific table(s) in all databases\n"),
 					progname);
 			exit(1);
 		}
 
-		vacuum_all_databases(full, verbose, analyze, freeze,
-							 host, port, username, prompt_password,
-							 progname, echo, quiet);
+		vacuum_all_databases(full, verbose, and_analyze, analyze_only, freeze,
+							 maintenance_db, host, port, username,
+							 prompt_password, progname, echo, quiet);
 	}
 	else
 	{
@@ -170,9 +205,25 @@ main(int argc, char *argv[])
 				dbname = get_user_name(progname);
 		}
 
-		vacuum_one_database(dbname, full, verbose, analyze, freeze, table,
-							host, port, username, prompt_password,
-							progname, echo);
+		if (tables.head != NULL)
+		{
+			SimpleStringListCell *cell;
+
+			for (cell = tables.head; cell; cell = cell->next)
+			{
+				vacuum_one_database(dbname, full, verbose, and_analyze,
+									analyze_only,
+									freeze, cell->val,
+									host, port, username, prompt_password,
+									progname, echo);
+			}
+		}
+		else
+			vacuum_one_database(dbname, full, verbose, and_analyze,
+								analyze_only,
+								freeze, NULL,
+								host, port, username, prompt_password,
+								progname, echo);
 	}
 
 	exit(0);
@@ -180,8 +231,8 @@ main(int argc, char *argv[])
 
 
 static void
-vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
-					bool freeze, const char *table,
+vacuum_one_database(const char *dbname, bool full, bool verbose, bool and_analyze,
+					bool analyze_only, bool freeze, const char *table,
 					const char *host, const char *port,
 					const char *username, enum trivalue prompt_password,
 					const char *progname, bool echo)
@@ -192,28 +243,74 @@ vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
 
 	initPQExpBuffer(&sql);
 
-	appendPQExpBuffer(&sql, "VACUUM");
-	if (full)
-		appendPQExpBuffer(&sql, " FULL");
-	if (freeze)
-		appendPQExpBuffer(&sql, " FREEZE");
-	if (verbose)
-		appendPQExpBuffer(&sql, " VERBOSE");
-	if (analyze)
-		appendPQExpBuffer(&sql, " ANALYZE");
+	conn = connectDatabase(dbname, host, port, username, prompt_password,
+						   progname, echo, false);
+
+	if (analyze_only)
+	{
+		appendPQExpBuffer(&sql, "ANALYZE");
+		if (verbose)
+			appendPQExpBuffer(&sql, " VERBOSE");
+	}
+	else
+	{
+		appendPQExpBuffer(&sql, "VACUUM");
+		if (PQserverVersion(conn) >= 90000)
+		{
+			const char *paren = " (";
+			const char *comma = ", ";
+			const char *sep = paren;
+
+			if (full)
+			{
+				appendPQExpBuffer(&sql, "%sFULL", sep);
+				sep = comma;
+			}
+			if (freeze)
+			{
+				appendPQExpBuffer(&sql, "%sFREEZE", sep);
+				sep = comma;
+			}
+			if (verbose)
+			{
+				appendPQExpBuffer(&sql, "%sVERBOSE", sep);
+				sep = comma;
+			}
+			if (and_analyze)
+			{
+				appendPQExpBuffer(&sql, "%sANALYZE", sep);
+				sep = comma;
+			}
+			if (sep != paren)
+				appendPQExpBuffer(&sql, ")");
+		}
+		else
+		{
+			if (full)
+				appendPQExpBuffer(&sql, " FULL");
+			if (freeze)
+				appendPQExpBuffer(&sql, " FREEZE");
+			if (verbose)
+				appendPQExpBuffer(&sql, " VERBOSE");
+			if (and_analyze)
+				appendPQExpBuffer(&sql, " ANALYZE");
+		}
+	}
 	if (table)
-		appendPQExpBuffer(&sql, " %s", table);
+	{
+		appendPQExpBufferChar(&sql, ' ');
+		appendQualifiedRelation(&sql, table, conn, progname, echo);
+	}
 	appendPQExpBuffer(&sql, ";\n");
 
-	conn = connectDatabase(dbname, host, port, username, prompt_password, progname);
 	if (!executeMaintenanceCommand(conn, sql.data, echo))
 	{
 		if (table)
 			fprintf(stderr, _("%s: vacuuming of table \"%s\" in database \"%s\" failed: %s"),
-					progname, table, dbname, PQerrorMessage(conn));
+					progname, table, PQdb(conn), PQerrorMessage(conn));
 		else
 			fprintf(stderr, _("%s: vacuuming of database \"%s\" failed: %s"),
-					progname, dbname, PQerrorMessage(conn));
+					progname, PQdb(conn), PQerrorMessage(conn));
 		PQfinish(conn);
 		exit(1);
 	}
@@ -223,19 +320,23 @@ vacuum_one_database(const char *dbname, bool full, bool verbose, bool analyze,
 
 
 static void
-vacuum_all_databases(bool full, bool verbose, bool analyze, bool freeze,
+vacuum_all_databases(bool full, bool verbose, bool and_analyze, bool analyze_only,
+					 bool freeze, const char *maintenance_db,
 					 const char *host, const char *port,
 					 const char *username, enum trivalue prompt_password,
 					 const char *progname, bool echo, bool quiet)
 {
 	PGconn	   *conn;
 	PGresult   *result;
+	PQExpBufferData connstr;
 	int			i;
 
-	conn = connectDatabase("postgres", host, port, username, prompt_password, progname);
+	conn = connectMaintenanceDatabase(maintenance_db, host, port,
+									  username, prompt_password, progname, echo);
 	result = executeQuery(conn, "SELECT datname FROM pg_database WHERE datallowconn ORDER BY 1;", progname, echo);
 	PQfinish(conn);
 
+	initPQExpBuffer(&connstr);
 	for (i = 0; i < PQntuples(result); i++)
 	{
 		char	   *dbname = PQgetvalue(result, i, 0);
@@ -246,10 +347,16 @@ vacuum_all_databases(bool full, bool verbose, bool analyze, bool freeze,
 			fflush(stdout);
 		}
 
-		vacuum_one_database(dbname, full, verbose, analyze, freeze, NULL,
-							host, port, username, prompt_password,
+		resetPQExpBuffer(&connstr);
+		appendPQExpBuffer(&connstr, "dbname=");
+		appendConnStrVal(&connstr, PQgetvalue(result, i, 0));
+
+		vacuum_one_database(connstr.data, full, verbose, and_analyze,
+							analyze_only,
+						 freeze, NULL, host, port, username, prompt_password,
 							progname, echo);
 	}
+	termPQExpBuffer(&connstr);
 
 	PQclear(result);
 }
@@ -268,17 +375,19 @@ help(const char *progname)
 	printf(_("  -f, --full                      do full vacuuming\n"));
 	printf(_("  -F, --freeze                    freeze row transaction information\n"));
 	printf(_("  -q, --quiet                     don't write any messages\n"));
-	printf(_("  -t, --table='TABLE[(COLUMNS)]'  vacuum specific table only\n"));
+	printf(_("  -t, --table='TABLE[(COLUMNS)]'  vacuum specific table(s) only\n"));
 	printf(_("  -v, --verbose                   write a lot of output\n"));
-	printf(_("  -z, --analyze                   update optimizer hints\n"));
-	printf(_("  --help                          show this help, then exit\n"));
-	printf(_("  --version                       output version information, then exit\n"));
+	printf(_("  -V, --version                   output version information, then exit\n"));
+	printf(_("  -z, --analyze                   update optimizer statistics\n"));
+	printf(_("  -Z, --analyze-only              only update optimizer statistics\n"));
+	printf(_("  -?, --help                      show this help, then exit\n"));
 	printf(_("\nConnection options:\n"));
 	printf(_("  -h, --host=HOSTNAME       database server host or socket directory\n"));
 	printf(_("  -p, --port=PORT           database server port\n"));
 	printf(_("  -U, --username=USERNAME   user name to connect as\n"));
 	printf(_("  -w, --no-password         never prompt for password\n"));
 	printf(_("  -W, --password            force password prompt\n"));
+	printf(_("  --maintenance-db=DBNAME   alternate maintenance database\n"));
 	printf(_("\nRead the description of the SQL command VACUUM for details.\n"));
 	printf(_("\nReport bugs to <pgsql-bugs@postgresql.org>.\n"));
 }
