@@ -3,10 +3,10 @@
  * geqo_pool.h
  *	  pool representation in optimizer/geqo
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
- * $Id: geqo_pool.h,v 1.17 2003/08/04 02:40:14 momjian Exp $
+ * src/include/optimizer/geqo_pool.h
  *
  *-------------------------------------------------------------------------
  */
@@ -23,19 +23,18 @@
 #ifndef GEQO_POOL_H
 #define GEQO_POOL_H
 
-#include "optimizer/geqo_gene.h"
-#include "nodes/parsenodes.h"
+#include "optimizer/geqo.h"
 
-extern Pool *alloc_pool(int pool_size, int string_length);
-extern void free_pool(Pool *pool);
 
-extern void random_init_pool(Query *root, List *initial_rels,
-				 Pool *pool, int strt, int stop);
-extern Chromosome *alloc_chromo(int string_length);
-extern void free_chromo(Chromosome *chromo);
+extern Pool *alloc_pool(PlannerInfo *root, int pool_size, int string_length);
+extern void free_pool(PlannerInfo *root, Pool *pool);
 
-extern void spread_chromo(Chromosome *chromo, Pool *pool);
+extern void random_init_pool(PlannerInfo *root, Pool *pool);
+extern Chromosome *alloc_chromo(PlannerInfo *root, int string_length);
+extern void free_chromo(PlannerInfo *root, Chromosome *chromo);
 
-extern void sort_pool(Pool *pool);
+extern void spread_chromo(PlannerInfo *root, Chromosome *chromo, Pool *pool);
+
+extern void sort_pool(PlannerInfo *root, Pool *pool);
 
 #endif   /* GEQO_POOL_H */
