@@ -1,8 +1,9 @@
-/* $PostgreSQL: pgsql/src/interfaces/ecpg/pgtypeslib/common.c,v 1.14 2006/07/30 16:28:58 meskes Exp $ */
+/* src/interfaces/ecpg/pgtypeslib/common.c */
 
 #include "postgres_fe.h"
 
 #include "extern.h"
+#include "pgtypes.h"
 
 /* Return value is zero-filled. */
 char *
@@ -16,7 +17,7 @@ pgtypes_alloc(long size)
 }
 
 char *
-pgtypes_strdup(char *str)
+pgtypes_strdup(const char *str)
 {
 	char	   *new = (char *) strdup(str);
 
@@ -42,10 +43,8 @@ pgtypes_fmt_replace(union un_fmt_comb replace_val, int replace_type, char **outp
 			i = strlen(replace_val.str_val);
 			if (i + 1 <= *pstr_len)
 			{
-				/*
-				 * copy over i + 1 bytes, that includes the tailing terminator
-				 */
-				strncpy(*output, replace_val.str_val, i + 1);
+				/* include trailing terminator in what we copy */
+				memcpy(*output, replace_val.str_val, i + 1);
 				*pstr_len -= i;
 				*output += i;
 				if (replace_type == PGTYPES_TYPE_STRING_MALLOCED)
@@ -138,3 +137,12 @@ pgtypes_fmt_replace(union un_fmt_comb replace_val, int replace_type, char **outp
 	}
 	return 0;
 }
+
+/* Functions declared in pgtypes.h. */
+
+/* Just frees memory (mostly needed for Windows) */
+void
+PGTYPESchar_free(char *ptr)
+{
+	free(ptr);
+}
