@@ -2,12 +2,12 @@
  *
  * isinf.c
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/port/isinf.c,v 1.13 2009/01/01 17:24:04 momjian Exp $
+ *	  src/port/isinf.c
  *
  *-------------------------------------------------------------------------
  */
