@@ -1,13 +1,13 @@
 /*-------------------------------------------------------------------------
  *
  * noblock.c
- *	  set a file descriptor as non-blocking
+ *	  set a file descriptor as blocking or non-blocking
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/port/noblock.c,v 1.13 2009/01/01 17:24:04 momjian Exp $
+ *	  src/port/noblock.c
  *
  *-------------------------------------------------------------------------
  */
@@ -17,32 +17,48 @@
 #include <fcntl.h>
 
 
+/*
+ * Put socket into nonblock mode.
+ * Returns true on success, false on failure.
+ */
 bool
-pg_set_noblock(int sock)
-{
-#if !defined(WIN32)
-	return (fcntl(sock, F_SETFL, O_NONBLOCK) != -1);
-#else
-	long		ioctlsocket_ret = 1;
-
-	/* Returns non-0 on failure, while fcntl() returns -1 on failure */
-	return (ioctlsocket(sock, FIONBIO, &ioctlsocket_ret) == 0);
-#endif
-}
-
-
-bool
-pg_set_block(int sock)
+pg_set_noblock(pgsocket sock)
 {
 #if !defined(WIN32)
 	int			flags;
 
 	flags = fcntl(sock, F_GETFL);
-	if (flags < 0 || fcntl(sock, F_SETFL, (long) (flags & ~O_NONBLOCK)))
+	if (flags < 0)
+		return false;
+	if (fcntl(sock, F_SETFL, (flags | O_NONBLOCK)) == -1)
 		return false;
 	return true;
 #else
-	long		ioctlsocket_ret = 0;
+	unsigned long ioctlsocket_ret = 1;
+
+	/* Returns non-0 on failure, while fcntl() returns -1 on failure */
+	return (ioctlsocket(sock, FIONBIO, &ioctlsocket_ret) == 0);
+#endif
+}
+
+/*
+ * Put socket into blocking mode.
+ * Returns true on success, false on failure.
+ */
+bool
+pg_set_block(pgsocket sock)
+{
+#if !defined(WIN32)
+	int			flags;
+
+	flags = fcntl(sock, F_GETFL);
+	if (flags < 0)
+		return false;
+	if (fcntl(sock, F_SETFL, (flags & ~O_NONBLOCK)) == -1)
+		return false;
+	return true;
+#else
+	unsigned long ioctlsocket_ret = 0;
 
 	/* Returns non-0 on failure, while fcntl() returns -1 on failure */
 	return (ioctlsocket(sock, FIONBIO, &ioctlsocket_ret) == 0);
