@@ -1,7 +1,10 @@
 /*
+ * src/test/examples/testlibpq.c
+ *
+ *
  * testlibpq.c
  *
- *		Test the C version of LIBPQ, the POSTGRES frontend library.
+ *		Test the C version of libpq, the PostgreSQL frontend library.
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,14 +29,13 @@ main(int argc, char **argv)
 
 	/*
 	 * If the user supplies a parameter on the command line, use it as the
-	 * conninfo string; otherwise default to setting dbname=template1 and
-	 * using environment variables or defaults for all other connection
-	 * parameters.
+	 * conninfo string; otherwise default to setting dbname=postgres and using
+	 * environment variables or defaults for all other connection parameters.
 	 */
 	if (argc > 1)
 		conninfo = argv[1];
 	else
-		conninfo = "dbname = template1";
+		conninfo = "dbname = postgres";
 
 	/* Make a connection to the database */
 	conn = PQconnectdb(conninfo);
@@ -41,16 +43,32 @@ main(int argc, char **argv)
 	/* Check to see that the backend connection was successfully made */
 	if (PQstatus(conn) != CONNECTION_OK)
 	{
-		fprintf(stderr, "Connection to database '%s' failed.\n", PQdb(conn));
-		fprintf(stderr, "%s", PQerrorMessage(conn));
+		fprintf(stderr, "Connection to database failed: %s",
+				PQerrorMessage(conn));
+		exit_nicely(conn);
+	}
+
+	/* Set always-secure search path, so malicous users can't take control. */
+	res = PQexec(conn,
+				 "SELECT pg_catalog.set_config('search_path', '', false)");
+	if (PQresultStatus(res) != PGRES_TUPLES_OK)
+	{
+		fprintf(stderr, "SET failed: %s", PQerrorMessage(conn));
+		PQclear(res);
 		exit_nicely(conn);
 	}
 
 	/*
-	 * Our test case here involves using a cursor, for which we must be
-	 * inside a transaction block.	We could do the whole thing with a
-	 * single PQexec() of "select * from pg_database", but that's too
-	 * trivial to make a good example.
+	 * Should PQclear PGresult whenever it is no longer needed to avoid memory
+	 * leaks
+	 */
+	PQclear(res);
+
+	/*
+	 * Our test case here involves using a cursor, for which we must be inside
+	 * a transaction block.  We could do the whole thing with a single
+	 * PQexec() of "select * from pg_database", but that's too trivial to make
+	 * a good example.
 	 */
 
 	/* Start a transaction block */
@@ -61,11 +79,6 @@ main(int argc, char **argv)
 		PQclear(res);
 		exit_nicely(conn);
 	}
-
-	/*
-	 * Should PQclear PGresult whenever it is no longer needed to avoid
-	 * memory leaks
-	 */
 	PQclear(res);
 
 	/*
