@@ -2,7 +2,7 @@
 **		entab.c			- add tabs to a text file
 **		by Bruce Momjian (root@candle.pha.pa.us)
 **
-** $PostgreSQL: pgsql/src/tools/entab/entab.c,v 1.16 2006/03/11 04:38:41 momjian Exp $
+** src/tools/entab/entab.c
 **
 **	version 1.3
 **
@@ -14,6 +14,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
+#include <unistd.h>
 
 #if defined(WIN32) || defined(__CYGWIN__)
 #define PG_BINARY_R "rb"
@@ -102,13 +103,13 @@ main(int argc, char **argv)
 		else
 		{
 			if ((in_file = fopen(*argv, PG_BINARY_R)) == NULL)
-				halt("PERROR:  Can not open file %s\n", argv[0]);
+				halt("PERROR:  Cannot open file %s\n", argv[0]);
 			argv++;
 		}
 
 		escaped = FALSE;
 
-		while (fgets(in_line, BUFSIZ, in_file) != NULL)
+		while (fgets(in_line, sizeof(in_line), in_file) != NULL)
 		{
 			col_in_tab = 0;
 			prv_spaces = 0;
@@ -181,10 +182,12 @@ main(int argc, char **argv)
 						if (*src == '\\')
 							escaped = TRUE;
 						if (*src == '"' || *src == '\'')
+						{
 							if (quote_char == ' ')
 								quote_char = *src;
 							else if (*src == quote_char)
 								quote_char = ' ';
+						}
 					}
 					else if (*src != '\r' && *src != '\n')
 						escaped = FALSE;
